"""Main views of the the Application (without opening a new dialog)"""

from .about import AboutPage
from .app_grid import AppGridView
from .conan_conf import ConanConfigView
from .conan_search import ConanSearchView
from .package_explorer import LocalConanPackageExplorer
from .plugins_manager import PluginsPage

__all__ = [
    "AppGridView",
    "LocalConanPackageExplorer",
    "ConanSearchView",
    "ConanConfigView",
    "AboutPage",
    "PluginsPage",
]



from typing import List, Optional

from PySide6.QtCore import QItemSelectionModel, QModelIndex, Qt, SignalInstance
from PySide6.QtWidgets import QApplication, QDialog, QListWidgetItem, QStyle, QTreeView

import conan_explorer.app as app
from conan_explorer.app.logger import Logger
from conan_explorer.conan_wrapper.types import Remote
from conan_explorer.ui.dialogs import QuestionWithItemListDialog, ReorderController

from .remotes_model import RemotesModelItem, RemotesTableModel


class ConanRemoteController:
    def __init__(
        self, view: QTreeView, conan_remotes_updated: Optional[SignalInstance]
    ) -> None:
        self._view = view
        self._model = RemotesTableModel()
        self.conan_remotes_updated = conan_remotes_updated

    def update(self):
        # save selected remote, if triggering a re-init
        sel_remote = self.get_selected_remote()
        self._remote_reorder_controller = ReorderController(self._view, self._model)

        self._model.setup_model_data()
        self._view.setItemsExpandable(False)
        self._view.setRootIsDecorated(False)
        self._view.setModel(self._model)
        self._view.expandAll()
        self.resize_remote_columns()

        if sel_remote:
            self._select_remote(sel_remote.name)
        if self.conan_remotes_updated:
            self.conan_remotes_updated.emit()

    def resize_remote_columns(self):
        for i in reversed(range(self._model.root_item.column_count() - 1)):
            self._view.resizeColumnToContents(i)
        # TODO calculate, if we need to make the name smaller
        self._view.setColumnWidth(1, 400)
        self._view.columnViewportPosition(0)

    def _select_remote(self, remote_name: str) -> bool:
        """Selects a remote in the view and returns true if it exists."""
        row_remote_to_sel = -1
        row = 0
        remote_item = None
        for row, remote_item in enumerate(self._model.items()):
            if remote_item.name == remote_name:
                row_remote_to_sel = row
                break
        if row_remote_to_sel < 0:
            Logger().debug("No remote to select")
            return False
        sel_model = self._view.selectionModel()
        sel_model.clearSelection()
        for column in range(self._model.columnCount(QModelIndex())):
            index = self._model.index(row_remote_to_sel, column, QModelIndex())
            sel_model.select(index, QItemSelectionModel.SelectionFlag.Select)
        return True

    def move_up(self):
        self._remote_reorder_controller.move_up()

    def move_down(self):
        self._remote_reorder_controller.move_down()

    def move_to_top(self):
        self._remote_reorder_controller.move_to_position(0)

    def move_to_bottom(self):
        self._remote_reorder_controller.move_to_position(-1)

    def add(self, remote: Remote):
        self._model.add(remote)

    def rename(self, remote: Remote, new_name):
        self._model.rename(remote, new_name)

    def update_remote(self, remote: Remote):
        self._model.update(remote)

    def login_remotes(self, remotes: List[str], user: str, pwd: str):
        for remote in remotes:
            # will be canceled after the first error, so no lockout will occur,
            # because of multiple incorrect logins error is printed on the console
            try:
                self._model.update_login_info(remote, user, pwd)
            except Exception as e:
                Logger().error(f"Can't sign in to {remote}: {str(e)}")
                return
            Logger().info(f"Successfully logged in to {remote}")

    def on_remove(self):
        remote_items = self.get_selected_remotes()
        dialog = QuestionWithItemListDialog(
            self._view, QStyle.StandardPixmap.SP_MessageBoxWarning
        )
        dialog.setWindowTitle("Delete remote(s)")
        dialog.set_question_text("Are you sure, you want to delete these remotes?\t")
        for item in remote_items:
            list_item = QListWidgetItem(item.name)
            list_item.setCheckState(Qt.CheckState.Checked)
            dialog.item_list_widget.addItem(list_item)

        reply = dialog.exec()
        if reply == QDialog.DialogCode.Accepted:
            # get user Selection
            for list_row in range(dialog.item_list_widget.count()):
                list_item = dialog.item_list_widget.item(list_row)
                if list_item.checkState() == Qt.CheckState.Unchecked:
                    continue
                self._model.remove(list_item.text())

    def remote_disable(self):
        remote_items = self.get_selected_remotes()
        if not remote_items:
            return
        for remote_item in remote_items:
            app.conan_api.disable_remote(remote_item.name, not remote_item.disabled)
        self.update()

    def copy_remote_name(self):
        remote_item = self.get_selected_remote()
        if not remote_item:
            return
        QApplication.clipboard().setText(remote_item.name)

    def get_selected_remote(self) -> Optional[Remote]:
        indexes = self._view.selectedIndexes()
        if len(indexes) == 0:  # can be multiple - always get 0
            Logger().debug("No selected item for context action")
            return None
        remote_model: RemotesModelItem = indexes[0].internalPointer()  # type: ignore
        for remote in app.conan_api.get_remotes(include_disabled=True):
            if remote.name == remote_model.name:
                return remote

    def get_selected_remotes(self) -> List[Remote]:
        indexes = self._view.selectedIndexes()
        remotes: List[Remote] = []

        for index in indexes:
            if index.column() != 0:  # we only need a row once
                continue
            remote_model: RemotesModelItem = index.internalPointer()  # type: ignore
            for remote in app.conan_api.get_remotes(include_disabled=True):
                if remote.name == remote_model.name:
                    remotes.append(remote)
        return remotes



from typing import Optional

from PySide6.QtCore import QSize, Qt, Signal, SignalInstance
from PySide6.QtWidgets import QComboBox, QDialog, QMessageBox, QTreeWidgetItem, QWidget

import conan_explorer.app as app
from conan_explorer.app import LoaderGui
from conan_explorer.app.logger import Logger  # using global module pattern
from conan_explorer.conan_wrapper.conan_worker import ConanWorkerElement
from conan_explorer.conan_wrapper.types import ConanOptions, ConanPkg, ConanRef
from conan_explorer.settings import DEFAULT_INSTALL_PROFILE
from conan_explorer.ui.common import get_themed_asset_icon
from conan_explorer.ui.dialogs.pkg_diff.diff import PkgDiffDialog


class ConanInstallDialog(QDialog):
    MARK_AS_DEFAULT_INSTALL_PROFILE = " *"
    installation_failed: SignalInstance = Signal(str)  # type: ignore - conan_ref

    def __init__(
        self,
        parent: Optional[QWidget],
        conan_full_ref: str,
        pkg_installed_signal: Optional[SignalInstance] = None,
        lock_reference=False,
        capture_install_info=False,
    ):
        """
        conan_ref can be in full ref format with <ref>:<id>
        lock_reference disables conan reference editing.
        capture_install_info does not actually install, just return the selected info.
        """
        super().__init__(parent)
        from .conan_install_ui import Ui_Dialog

        self._ui = Ui_Dialog()
        self._ui.setupUi(self)
        self.pkg_installed_signal = pkg_installed_signal
        self._capture_install_info = capture_install_info
        self._conan_selected_install: Optional[ConanWorkerElement] = None
        self._pkg_diff_items = []
        # style
        icon = get_themed_asset_icon("icons/download_pkg.svg", True)
        self.setWindowIcon(icon)

        # init search bar
        self._ui.conan_ref_line_edit.validator_enabled = True
        self._ui.conan_ref_line_edit.setText(conan_full_ref)
        if lock_reference:
            self._ui.conan_ref_line_edit.setEnabled(False)

        # button box
        self._ui.button_box.accepted.connect(self.on_install)
        self.adjust_to_size()

        # hide items, when it has a ref:
        if ":" in conan_full_ref:
            self.hide_config_elements()
            self.setFixedHeight(175)
            return
        # profiles
        self.load_profiles()
        self.load_options(conan_full_ref)

        if capture_install_info:  # auto install makes no sense in this mode
            self.hide_for_install_info()
            self.resize(self.width(), self.height() - 150)

        # disable profile and options on activating this
        self._ui.auto_install_check_box.clicked.connect(self.on_auto_install_check)
        self._ui.set_default_install_profile_button.clicked.connect(
            self.on_set_default_install_profile
        )
        self.installation_failed.connect(self.on_installation_failed)

    def hide_config_elements(self):
        self._ui.profile_cbox.hide()
        self._ui.profile_label.hide()
        self._ui.set_default_install_profile_button.hide()
        self._ui.conan_opts_label.hide()
        self._ui.options_widget.hide()
        self._ui.line.hide()
        self._ui.auto_install_label.hide()
        self._ui.auto_install_check_box.hide()

    def hide_for_install_info(self):
        self._ui.auto_install_check_box.hide()
        self._ui.auto_install_label.hide()
        self._ui.update_label.hide()
        self._ui.update_check_box.hide()
        self._ui.line.hide()

    def load_profiles(self):
        self._ui.profile_cbox.clear()
        profiles = app.conan_api.get_profiles()
        default_profile = app.active_settings.get_string(DEFAULT_INSTALL_PROFILE)
        try:
            default_index = profiles.index(default_profile)
        except Exception:
            default_index = None
        if not default_index:
            app.active_settings.set(DEFAULT_INSTALL_PROFILE, "default")
        try:
            if default_index is not None:
                profiles.pop(default_index)
                profiles.insert(0, default_profile + self.MARK_AS_DEFAULT_INSTALL_PROFILE)
        except Exception:
            Logger().debug("Can't mark default install profile")
        self._ui.profile_cbox.addItems(profiles)

    def load_options(self, conan_full_ref: str):
        # options table
        default_options = {}
        conan_ref = ""
        try:
            conan_ref = conan_full_ref.split(":")[0]
            ConanRef.loads(conan_ref)
        except Exception:
            return
        loader = LoaderGui(self)
        loader.load(
            self, self.on_options_query, (conan_ref,), loading_text="Loading options..."
        )
        loader.wait_for_finished()
        options = loader.return_value
        if options is None:
            return
        available_options, default_options = options

        # doing this after connecting toggle_auto_install_on_pkg_ref initializes it correctly
        for name, value in default_options.items():
            item = QTreeWidgetItem(self._ui.options_widget)
            item.setData(0, 0, name)
            item.setData(1, 0, str(value))
            item.setFlags(Qt.ItemFlag.ItemIsEnabled)  # Qt.ItemFlag.ItemIsEditable |
            self._ui.options_widget.addTopLevelItem(item)
            try:
                values = available_options[name]
                if isinstance(values, list) and values != ["ANY"]:
                    cb = QComboBox()
                    values_str = [str(x) for x in values]
                    cb.addItems(values_str)
                    cb.setCurrentText(str(value))
                    self._ui.options_widget.setItemWidget(item, 1, cb)
            except Exception as e:
                Logger().debug("Error while gettings options: %s", str(e))
        self._ui.options_widget.resizeColumnToContents(1)
        self._ui.options_widget.resizeColumnToContents(0)
        self._ui.options_widget.itemDoubleClicked.connect(self.onTreeWidgetItemDoubleClicked)

    def onTreeWidgetItemDoubleClicked(self, item):
        self._ui.options_widget.openPersistentEditor(item, 1)

    def on_options_query(self, conan_ref: str):
        try:
            return app.conan_api.get_options_with_default_values(ConanRef.loads(conan_ref))
        except Exception:
            return {}, {}

    def on_auto_install_check(self):
        enabled = True
        if self._ui.auto_install_check_box.isChecked():
            enabled = False
        self._ui.profile_cbox.setEnabled(enabled)
        self._ui.options_widget.setEnabled(enabled)
        self._ui.update_check_box.setEnabled(enabled)

    def adjust_to_size(self):
        """Expands the dialog to the length of the install ref text.
        (Somehow the dialog sets a much smaller size then it should via expanding size policy and layout.)
        """
        self._ui.conan_ref_line_edit.adjustSize()
        self.adjustSize()
        h_offset = (self.size() - self._ui.conan_ref_line_edit.size()).width()  # type: ignore
        width = (
            self._ui.conan_ref_line_edit.fontMetrics()
            .boundingRect(self._ui.conan_ref_line_edit.text())
            .width()
        )
        if width < 250:
            width = 250
        self.resize(QSize(width + h_offset + 15, self.height()))  # 15 margin

    def on_install(self):
        ref_text = self._ui.conan_ref_line_edit.text()
        update_check_state = False
        if self._ui.update_check_box.checkState() == Qt.CheckState.Checked:
            update_check_state = True

        auto_install_checked = False
        options = {}
        if self._ui.auto_install_check_box.checkState() == Qt.CheckState.Checked:
            auto_install_checked = True
        else:
            # options from selection
            options = self.get_user_options()
        self._conan_selected_install = {
            "ref_pkg_id": ref_text,
            "settings": {},
            "profile": self.get_selected_profile(),
            "options": options,
            "update": update_check_state,
            "auto_install": auto_install_checked,
        }
        if not self._capture_install_info:
            app.conan_worker.put_ref_in_install_queue(
                self._conan_selected_install, self.emit_conan_pkg_signal_callback
            )

    def emit_conan_pkg_signal_callback(self, conan_ref: str, pkg_id: str):
        if not pkg_id:
            self.installation_failed.emit(conan_ref)
            return
        if not self.pkg_installed_signal:
            return
        self.pkg_installed_signal.emit(conan_ref, pkg_id)

    def on_installation_failed(self, conan_ref: str):
        # installation failed
        message_box = QMessageBox(parent=self)
        message_box.setWindowTitle("Compare packages")
        message_box.setText(
            (
                f"Installation of '{conan_ref}' failed.\n"
                "Do you want to compare to packages on the remotes?"
            )
        )
        sb = QMessageBox.StandardButton
        message_box.setIcon(QMessageBox.Icon.Question)
        message_box.setStandardButtons(sb.Yes | sb.Cancel)
        reply = message_box.exec()
        if reply == sb.Yes:
            self.show_package_diffs(conan_ref)
        message_box.close()

    def get_all_pkgs(self, conan_ref: str):
        return app.conan_api.get_remote_pkgs_from_ref(ConanRef.loads(conan_ref), "all")

    def show_package_diffs(self, conan_ref: str):
        try:
            loader = LoaderGui(self)
            loader.load(
                self, self.get_all_pkgs, (conan_ref,), loading_text="Getting all packages..."
            )
            loader.wait_for_finished()
            items = loader.return_value
            if not items:
                return
            if len(items) < 2:
                return
            if not self._conan_selected_install:
                return
            dialog = PkgDiffDialog(self.parent())
            installed_pkg_info: ConanPkg = {
                "id": "User input",
                "settings": app.conan_api.get_profile_settings(
                    self._conan_selected_install["profile"]
                ),
                "options": self._conan_selected_install["options"],
                "requires": [],
                "outdated": False,
            }
            dialog.add_diff_item(installed_pkg_info)
            for item in items:
                dialog.add_diff_item(item)
            dialog.show()
        except Exception as e:
            Logger().error(str(e))

    def on_set_default_install_profile(self):
        selected_profile = self.get_selected_profile()
        app.active_settings.set(DEFAULT_INSTALL_PROFILE, selected_profile)
        # re-select after clear
        self.load_profiles()
        self._ui.profile_cbox.setCurrentText(
            selected_profile + self.MARK_AS_DEFAULT_INSTALL_PROFILE
        )

    def get_selected_profile(self) -> str:
        selected_profile = self._ui.profile_cbox.currentText()
        return selected_profile.rstrip(self.MARK_AS_DEFAULT_INSTALL_PROFILE)

    def get_user_options(self) -> ConanOptions:
        options = {}
        self._ui.options_widget.updateEditorData()
        for i in range(0, self._ui.options_widget.topLevelItemCount()):
            item = self._ui.options_widget.topLevelItem(i)
            if not item:
                continue
            widget = self._ui.options_widget.itemWidget(item, 1)
            value = item.data(1, 0)
            if isinstance(widget, QComboBox):
                value = widget.currentText()
            options[item.data(0, 0)] = value
        return options

    def get_selected_install_info(self):
        return self._conan_selected_install



import configparser
import uuid
from pathlib import Path
from typing import List, Union

import conan_explorer.app as app
from conan_explorer import BUILT_IN_PLUGIN
from conan_explorer.app.logger import Logger
from conan_explorer.app.system import str2bool
from conan_explorer.settings import PLUGINS_SECTION_NAME
from conan_explorer.ui.plugin.types import PluginDescription


class PluginValidationException(Exception): ...


class PluginFile:
    """
    Plugin file related methods.
    Format implemented as ini file.
    """

    @staticmethod
    def register(plugin_file_path: Union[Path, str]):
        """Register the given file into the application settings with a uuid."""
        plugin_file_path = Path(plugin_file_path)
        # check, if path already registered
        for plugin_group_name in app.active_settings.get_settings_from_node(
            PLUGINS_SECTION_NAME
        ):
            settings_plugin_file_path = app.active_settings.get_string(plugin_group_name)
            if Path(settings_plugin_file_path) == plugin_file_path:
                return
        app.active_settings.add(str(uuid.uuid1()), str(plugin_file_path), PLUGINS_SECTION_NAME)

    @staticmethod
    def unregister(plugin_file_path: Union[Path, str]):
        """Unregister given plugin file from application settings"""
        plugin_file_path = Path(plugin_file_path)
        for plugin_group_name in app.active_settings.get_settings_from_node(
            PLUGINS_SECTION_NAME
        ):
            settings_plugin_file_path = app.active_settings.get_string(plugin_group_name)
            if Path(settings_plugin_file_path) == plugin_file_path:
                app.active_settings.remove(plugin_group_name)

    @staticmethod
    def read(plugin_file_path: Union[Path, str]) -> List[PluginDescription]:
        """Read given plugin file and return list of contained plugin descriptions."""
        plugin_file_path = Path(plugin_file_path)
        if not plugin_file_path.is_file():
            Logger().error(f"Plugin file {plugin_file_path} does not exist.")
            return []  # error
        plugins = []
        parser = configparser.ConfigParser()
        parser.read(plugin_file_path, encoding="utf-8")
        PVE = PluginValidationException
        for section in parser.keys():
            if parser.default_section == section:
                continue
            plugin_info = parser[section]
            try:
                name = plugin_info.get("name")
                if not name:
                    raise PVE("field 'name' is required")
                version = plugin_info.get("version", "Unknown")

                icon_str = plugin_info.get("icon")
                if not icon_str:
                    raise PVE("field 'icon' is required")
                if version == BUILT_IN_PLUGIN:
                    icon = icon_str
                else:
                    icon_path = plugin_file_path.parent / icon_str
                    if not icon_path.is_file():
                        raise PVE(f"icon {str(icon_path)} does not exist.")
                    icon = str(icon_path)

                if not plugin_info.get("import_path"):
                    raise PVE("field 'import_path' is required")
                import_path = plugin_file_path.parent / plugin_info.get("import_path")
                if not import_path.is_dir():
                    raise PVE(f"import_path {str(import_path)} does not exist.")
                if import_path.is_dir():  # needs an __init__.py
                    init_py_path = import_path / "__init__.py"
                    init_pyc_path = import_path / "__init__.pyc"  # for precompiled
                    if not (init_py_path.exists() or init_pyc_path.exists()):
                        raise PVE(f"{str(init_py_path)} does not exist")

                plugin_class = plugin_info.get("plugin_class")
                if not plugin_class:
                    raise PVE("field 'plugin_class' is required")
                description = plugin_info.get("description", "")
                author = plugin_info.get("author", "Unknown")
                side_menu = str2bool(plugin_info.get("side_menu", "False"))
                conan_versions = plugin_info.get("conan_versions", "")
                desc = PluginDescription(
                    name,
                    version,
                    author,
                    icon,
                    str(import_path),
                    plugin_class,
                    description,
                    side_menu,
                    conan_versions,
                )
                plugins.append(desc)
            except Exception as e:
                Logger().error(
                    f"Can't read {section} plugin information from {plugin_file_path}: {str(e)}."
                )

        return plugins

    @staticmethod
    def write(plugin_file_path: Union[Path, str], plugin_descriptions: List[PluginDescription]):
        """Write plugin file to specified path with the given the plugin descripton"""
        parser = configparser.ConfigParser()
        for i, description in enumerate(plugin_descriptions):
            section_name = "PluginDescription" + str(i)
            parser.add_section(section_name)
            for setting, value in description.__dict__.items():
                parser[section_name][setting] = str(value)
        with open(plugin_file_path, "w", encoding="utf-8") as fd:
            parser.write(fd)



from pathlib import Path
from typing import List

from PySide6.QtCore import QModelIndex, Qt
from PySide6.QtGui import QColor, QFont
from typing_extensions import override

from conan_explorer.ui.common import FileSystemModel
from conan_explorer.ui.common.model import QAF


class CalFileSystemModel(FileSystemModel):
    def __init__(
        self, h_align=QAF.AlignLeft | QAF.AlignVCenter, v_align=QAF.AlignVCenter, parent=None
    ):
        super().__init__(h_align, v_align, parent)
        self._disabled_rows: "set[int]" = set()

    @override
    def data(self, index: QModelIndex, role: Qt.ItemDataRole):
        if role == Qt.ItemDataRole.FontRole:
            if self._row_is_disabled(index):
                font = QFont()
                font.setItalic(True)
                return font
        if role == Qt.ItemDataRole.ForegroundRole:
            try:
                if self._row_is_disabled(index):
                    return QColor(Qt.GlobalColor.gray)
            except Exception:
                pass
        return super().data(index, role)

    def _row_is_disabled(self, index: QModelIndex):
        return index.row() in self._disabled_rows

    def add_disabled_items(self, item_paths: List[str]):
        for item_path in item_paths:
            self._disabled_rows.add(self.index(Path(item_path).as_posix(), 0).row())

    def clear_all_disabled_items(self):
        self._disabled_rows = set()

    def clear_disabled_item(self, item_path: str):
        try:
            disabled_item = self.index(Path(item_path).as_posix(), 0).row()
            self._disabled_rows.remove(disabled_item)
        except Exception:
            pass  # element not found



from pathlib import Path
from typing import Optional

from PySide6.QtWidgets import QDialog, QStyle, QTreeWidgetItem, QWidget

from conan_explorer.app import LoaderGui  # using global module pattern
from conan_explorer.app.logger import Logger
from conan_explorer.app.system import delete_path, get_folder_size_mb
from conan_explorer.conan_wrapper.conan_cleanup import ConanCleanup


class ConanCacheCleanupDialog(QDialog):
    def __init__(self, parent: Optional[QWidget]):
        super().__init__(parent)
        from .conan_cache_cleanup_ui import Ui_Dialog

        self._ui = Ui_Dialog()
        self._ui.setupUi(self)

        ref_items = []
        self.cleanup_info = None

        cleaner = ConanCleanup()
        loader = LoaderGui(self)
        self.loader = loader
        self.accepted.connect(self.on_accept)
        loader.load(
            self,
            cleaner.get_cleanup_cache_info,
            loading_text="Gathering obsolete directories...",
            cancel_button=False,
        )
        loader.wait_for_finished()
        cleanup_info = loader.return_value
        if not cleanup_info:
            Logger().info("Nothing found in cache to clean up.")
            return

        def setup_model_data_with_sizes(cleanup_info):
            size_mbytes = 0
            for ref, paths in cleanup_info.items():
                ref_item = QTreeWidgetItem([ref])
                # ref_item.setCheckState(0, Qt.CheckState.Checked)
                for path_type, path in paths.items():
                    loader.loading_string_signal.emit(
                        (
                            f"Calculating size of {ref} {path_type}\n"
                            f"Found {size_mbytes:.1f}MB to clean up"
                        )
                    )
                    size_mbytes_item = get_folder_size_mb(Path(path))
                    child = QTreeWidgetItem([path_type, f"{size_mbytes_item:.2f}"])
                    # child.setCheckState(0, Qt.CheckState.Checked)
                    size_mbytes += size_mbytes_item
                    ref_item.addChild(child)
                ref_items.append(ref_item)
            return size_mbytes

        loader.load(self, setup_model_data_with_sizes, (cleanup_info,), cancel_button=False)
        loader.wait_for_finished()
        size = loader.return_value
        self.cleanup_info = cleanup_info

        if size is None:
            size = 0

        self._ui.cleanup_tree_widget.addTopLevelItems(ref_items)
        self._ui.question_label.setText(
            f"Found {size:.2f} MB to clean up.\n"
            "Are you sure you want to delete the found folders?\t"
        )
        pixmapi = getattr(QStyle, "SP_MessageBoxQuestion")
        if pixmapi:
            icon = self.style().standardIcon(pixmapi)
            self._ui.icon.setPixmap(icon.pixmap(40, 40))
        self._ui.cleanup_tree_widget.expandAll()
        self._ui.cleanup_tree_widget.resizeColumnToContents(0)

        self.show()

    def on_accept(self):
        pass
        if not self.cleanup_info:
            return

        def delete_cache_paths(cleanup_info):
            #     for item in self._ui.cleanup_tree_widget.selectedItems():
            #         for
            for ref, paths in cleanup_info.items():
                for type, path in paths.items():
                    self.loader.loading_string_signal.emit("Deleting\n" + str(path))
                    delete_path(Path(path))

        self.loader.load(
            self,
            delete_cache_paths,
            (self.cleanup_info,),
            loading_text="Deleting cache paths...",
            cancel_button=False,
        )



import platform
from pathlib import Path
from typing import Dict, List, Set

import conan_explorer.app as app
from conan_explorer import conan_version
from conan_explorer.app.logger import Logger
from conan_explorer.conan_wrapper.types import ConanRef


class ConanCleanup:
    def __init__(self) -> None:
        self.cleanup_refs_info: Dict[str, Dict[str, str]] = {}
        self.invalid_metadata_refs: Set[str] = set()

    def gather_invalid_remote_metadata(self) -> List[str]:
        """Gather all references with invalid remotes"""
        invalid_refs = []
        remotes = app.conan_api.get_remotes(include_disabled=True)
        remote_names = [r.name for r in remotes]
        for ref in app.conan_api.get_all_local_refs():
            # This will not updated to the unified API - only V1 relevant
            ref_cache = app.conan_api._client_cache.package_layout(ref)  # type: ignore
            ref_remote = ""
            try:
                ref_remote = ref_cache.load_metadata().recipe.remote  # type: ignore
            except Exception:
                Logger().debug("Can't load metadata for %s", str(ref), exc_info=True)
                continue
            if ref_remote not in remote_names:
                invalid_refs.append(str(ref))
        self.invalid_metadata_refs = set(invalid_refs)
        return invalid_refs

    def repair_invalid_remote_metadata(self, invalid_ref):
        """Repair all references with invalid remotes"""
        # calling inspect with a correct remote repairs the metadata
        for remote in app.conan_api.get_remotes():
            try:
                app.conan_api._conan.inspect(invalid_ref, None, remote.name)  # type: ignore
                break
            except Exception:
                continue

    def get_cleanup_cache_info(self) -> Dict[str, Dict[str, str]]:
        """Get a list of orphaned short path and cache folders"""
        if platform.system() != "Windows" or conan_version.major == 2:
            return {}  # TODO: Update for linux
        from .types import CONAN_LINK

        editables = app.conan_api.get_editable_references()

        for conan_ref in app.conan_api.get_all_local_refs():
            if conan_ref in editables:
                continue
            current_ref_info = {}

            # this will not updated to the unified API - only V1 relevant
            ref_cache = app.conan_api._client_cache.package_layout(conan_ref)  # type: ignore

            # get orphaned refs
            package_ids = []
            try:
                package_ids = ref_cache.package_ids()
            except Exception:
                try:
                    # old API of Conan
                    package_ids = ref_cache.packages_ids()  # type: ignore
                except Exception as e:
                    Logger().debug(
                        "Cannot check pkg id for %s: %s", str(conan_ref), str(e), exc_info=True
                    )

            for pkg_id in package_ids:
                short_path_dir = app.conan_api.get_package_folder(conan_ref, pkg_id)
                if not short_path_dir.exists():
                    current_ref_info[str(pkg_id)] = str(Path(ref_cache.packages()) / pkg_id)

            # get temporary dirs
            source_path = Path(ref_cache.source())
            if source_path.exists():
                # check for .conan_link
                if (source_path / CONAN_LINK).is_file():
                    path = (source_path / CONAN_LINK).read_text()
                    current_ref_info["source"] = path.strip()
                else:
                    current_ref_info["source"] = ref_cache.source()

            if Path(ref_cache.builds()).exists():
                current_ref_info["build"] = ref_cache.builds()

            scm_source_path = Path(ref_cache.scm_sources())
            if scm_source_path.exists():
                # check for .conan_link
                if (scm_source_path / CONAN_LINK).is_file():
                    path = (scm_source_path / CONAN_LINK).read_text()
                    current_ref_info["scm_source"] = path.strip()
                else:
                    current_ref_info["scm_source"] = ref_cache.scm_sources()
            download_folder = Path(ref_cache.base_folder()) / "dl"
            if download_folder.exists():
                current_ref_info["download"] = str(download_folder)

            if current_ref_info:
                self.cleanup_refs_info[str(conan_ref)] = current_ref_info

        self.find_orphaned_packages()
        self.cleanup_refs_info = dict(sorted(self.cleanup_refs_info.items()))
        return self.cleanup_refs_info

    def find_orphaned_packages(self):
        """Reverse search for orphaned packages on windows short paths"""
        if platform.system() != "Windows" or conan_version.major == 2:
            return {}
        from .types import CONAN_REAL_PATH

        short_path_folders = [
            f for f in app.conan_api.get_short_path_root().iterdir() if f.is_dir()
        ]
        for short_path in short_path_folders:
            rp_file = short_path / CONAN_REAL_PATH
            if not rp_file.is_file():
                continue
            real_path = rp_file.read_text()

            if Path(real_path).is_dir():
                continue
            conan_ref = "Unknown"
            info_type = "Unknown"
            try:
                # try to reconstruct conan ref from real_path
                rel_path = Path(real_path).relative_to(app.conan_api.get_storage_path())
                info_type = rel_path.parts[4]
                conan_ref = str(
                    ConanRef(
                        rel_path.parts[0],
                        rel_path.parts[1],
                        rel_path.parts[2],
                        rel_path.parts[3],
                    )
                )

            except Exception:
                Logger().error(f"Can't read {CONAN_REAL_PATH} in {str(short_path)}")
            if not self.cleanup_refs_info.get(conan_ref):
                self.cleanup_refs_info[conan_ref] = {}
            self.cleanup_refs_info[conan_ref][info_type] = str(short_path)




from pathlib import Path
from typing import Optional

from PySide6.QtWidgets import QDialog, QFileDialog, QWidget
from conan_explorer.conan_wrapper.types import EditablePkg

from conan_explorer.ui.common.theming import get_themed_asset_icon
from conan_explorer.ui.views.conan_conf.editable_controller import ConanEditableController

class EditableEditDialog(QDialog):
    def __init__(self, editable: Optional[EditablePkg], editable_controller: ConanEditableController, parent: Optional[QWidget] = None):
        super().__init__(parent=parent)
        from .editable_edit_dialog_ui import Ui_Dialog
        if editable is None:
            editable = EditablePkg("", "", "")
        self._editable = editable
        self._editable_controller = editable_controller
        self._ui = Ui_Dialog()
        self._ui.setupUi(self)

        self.setWindowIcon(get_themed_asset_icon("icons/edit.svg", True))
        self._ui.button_box.accepted.connect(self.save)
        self._ui.path_browse_button.clicked.connect(self.on_path_browse_button_clicked)
        self._ui.output_folder_browse_button.clicked.connect(self.on_output_browse_button_clicked)

        self._ui.name_line_edit.setText(editable.conan_ref)
        self._ui.path_line_edit.setText(editable.path)
        self._ui.output_folder_line_edit.setText(str(editable.output_folder))

    def on_path_browse_button_clicked(self):
        current_path = self._ui.path_line_edit.text()
        default_path = current_path if current_path else str(Path.home())
        path = self.select_folder_dialog(default_path)
        if not path:
            return
        self._ui.path_line_edit.setText(path)

    def on_output_browse_button_clicked(self):
        current_path = self._ui.output_folder_line_edit.text()
        default_path = current_path if current_path else str(Path.home())
        path = self.select_folder_dialog(default_path)
        if not path:
            return
        self._ui.output_folder_line_edit.setText(path)

    def select_folder_dialog(self, start_path: str) -> str:
        dialog = QFileDialog(parent=self, caption="Select path",
                             directory=str(start_path),)
        dialog.setFileMode(QFileDialog.FileMode.Directory)
        dialog.setOption(QFileDialog.Option.ShowDirsOnly)
        if dialog.exec() == QFileDialog.DialogCode.Accepted:
            return dialog.selectedFiles()[0]
        return ""

    def save(self):
        """ Save edited editable information by calling the appropriate conan methods. """
        new_name = self._ui.name_line_edit.text().strip()
        remove_after_add = False # if name changed -> remove the old one
        if self._editable.conan_ref and self._editable.conan_ref != new_name:
            remove_after_add = True

        new_editable = EditablePkg(new_name, self._ui.path_line_edit.text().strip(), 
                                   self._ui.output_folder_line_edit.text())
        if self._editable_controller.add(new_editable):
            if remove_after_add:
                self._editable_controller.remove(self._editable)
        self.accept()



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'conan_search.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QAbstractScrollArea, QApplication, QFrame,
    QHBoxLayout, QHeaderView, QLabel, QLayout,
    QListView, QListWidget, QListWidgetItem, QPushButton,
    QSizePolicy, QSplitter, QTextBrowser, QToolButton,
    QTreeView, QVBoxLayout, QWidget)

from conan_explorer.ui.widgets.conan_line_edit import ConanRefLineEdit

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(432, 391)
        Form.setStyleSheet(u"")
        self.verticalLayout = QVBoxLayout(Form)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(2, 1, 1, 2)
        self.search_text_button_layout = QHBoxLayout()
        self.search_text_button_layout.setObjectName(u"search_text_button_layout")
        self.search_text_button_layout.setSizeConstraint(QLayout.SizeConstraint.SetDefaultConstraint)
        self.search_line = ConanRefLineEdit(Form)
        self.search_line.setObjectName(u"search_line")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.search_line.sizePolicy().hasHeightForWidth())
        self.search_line.setSizePolicy(sizePolicy)
        self.search_line.setMinimumSize(QSize(100, 0))
        self.search_line.setMaximumSize(QSize(16777215, 16777215))
        self.search_line.setSizeIncrement(QSize(0, 0))
        self.search_line.setBaseSize(QSize(500, 0))
        self.search_line.setLayoutDirection(Qt.LayoutDirection.RightToLeft)
        self.search_line.setClearButtonEnabled(True)

        self.search_text_button_layout.addWidget(self.search_line)

        self.search_button = QPushButton(Form)
        self.search_button.setObjectName(u"search_button")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.search_button.sizePolicy().hasHeightForWidth())
        self.search_button.setSizePolicy(sizePolicy1)
        self.search_button.setMinimumSize(QSize(150, 0))
        self.search_button.setMaximumSize(QSize(150, 16777215))
        icon = QIcon()
        icon.addFile(u"../../../assets/icons/search_packages.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.search_button.setIcon(icon)

        self.search_text_button_layout.addWidget(self.search_button)

        self.install_button = QPushButton(Form)
        self.install_button.setObjectName(u"install_button")
        sizePolicy1.setHeightForWidth(self.install_button.sizePolicy().hasHeightForWidth())
        self.install_button.setSizePolicy(sizePolicy1)
        self.install_button.setMinimumSize(QSize(150, 0))
        self.install_button.setMaximumSize(QSize(150, 16777215))
        icon1 = QIcon()
        icon1.addFile(u"../../../assets/icons/download_pkg.png", QSize(), QIcon.Mode.Normal, QIcon.State.Off)
        self.install_button.setIcon(icon1)

        self.search_text_button_layout.addWidget(self.install_button)

        self.search_text_button_layout.setStretch(0, 1)
        self.search_text_button_layout.setStretch(1, 1)
        self.search_text_button_layout.setStretch(2, 1)

        self.verticalLayout.addLayout(self.search_text_button_layout)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.remote_toggle_button = QToolButton(Form)
        self.remote_toggle_button.setObjectName(u"remote_toggle_button")
        self.remote_toggle_button.setStyleSheet(u" border: none; ")
        self.remote_toggle_button.setCheckable(True)
        self.remote_toggle_button.setToolButtonStyle(Qt.ToolButtonStyle.ToolButtonTextBesideIcon)
        self.remote_toggle_button.setArrowType(Qt.ArrowType.RightArrow)

        self.horizontalLayout.addWidget(self.remote_toggle_button)

        self.remote_hline = QFrame(Form)
        self.remote_hline.setObjectName(u"remote_hline")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Maximum)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.remote_hline.sizePolicy().hasHeightForWidth())
        self.remote_hline.setSizePolicy(sizePolicy2)
        self.remote_hline.setStyleSheet(u"")
        self.remote_hline.setFrameShape(QFrame.Shape.HLine)
        self.remote_hline.setFrameShadow(QFrame.Shadow.Sunken)
        self.remote_hline.setLineWidth(2)

        self.horizontalLayout.addWidget(self.remote_hline)


        self.verticalLayout.addLayout(self.horizontalLayout)

        self.frame = QFrame(Form)
        self.frame.setObjectName(u"frame")
        self.frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.frame.setFrameShadow(QFrame.Shadow.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.select_all_widget = QListWidget(self.frame)
        __qlistwidgetitem = QListWidgetItem(self.select_all_widget)
        __qlistwidgetitem.setCheckState(Qt.Checked);
        __qlistwidgetitem.setFlags(Qt.ItemIsSelectable|Qt.ItemIsDragEnabled|Qt.ItemIsUserCheckable|Qt.ItemIsEnabled|Qt.ItemIsAutoTristate);
        self.select_all_widget.setObjectName(u"select_all_widget")
        sizePolicy1.setHeightForWidth(self.select_all_widget.sizePolicy().hasHeightForWidth())
        self.select_all_widget.setSizePolicy(sizePolicy1)
        self.select_all_widget.setMaximumSize(QSize(16777215, 30))
        self.select_all_widget.setFrameShape(QFrame.Shape.NoFrame)
        self.select_all_widget.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)
        self.select_all_widget.setResizeMode(QListView.ResizeMode.Fixed)
        self.select_all_widget.setUniformItemSizes(True)

        self.verticalLayout_2.addWidget(self.select_all_widget)

        self.remote_list = QListWidget(self.frame)
        self.remote_list.setObjectName(u"remote_list")
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.remote_list.sizePolicy().hasHeightForWidth())
        self.remote_list.setSizePolicy(sizePolicy3)
        self.remote_list.setMinimumSize(QSize(0, 0))
        self.remote_list.setMaximumSize(QSize(16777215, 16777215))
        self.remote_list.setFrameShape(QFrame.Shape.NoFrame)
        self.remote_list.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.remote_list.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
        self.remote_list.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)
        self.remote_list.setAutoScroll(False)
        self.remote_list.setSelectionMode(QAbstractItemView.SelectionMode.ContiguousSelection)
        self.remote_list.setMovement(QListView.Movement.Snap)
        self.remote_list.setResizeMode(QListView.ResizeMode.Adjust)
        self.remote_list.setLayoutMode(QListView.LayoutMode.Batched)
        self.remote_list.setUniformItemSizes(True)
        self.remote_list.setSelectionRectVisible(False)
        self.remote_list.setSortingEnabled(True)

        self.verticalLayout_2.addWidget(self.remote_list)

        self.verticalLayout_2.setStretch(1, 1)

        self.verticalLayout.addWidget(self.frame)

        self.splitter = QSplitter(Form)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Orientation.Horizontal)
        self.splitter.setChildrenCollapsible(False)
        self.layoutWidget_2 = QWidget(self.splitter)
        self.layoutWidget_2.setObjectName(u"layoutWidget_2")
        self.results_layout = QVBoxLayout(self.layoutWidget_2)
        self.results_layout.setObjectName(u"results_layout")
        self.results_layout.setSizeConstraint(QLayout.SizeConstraint.SetMinAndMaxSize)
        self.results_layout.setContentsMargins(2, 0, 0, 0)
        self.results_label = QLabel(self.layoutWidget_2)
        self.results_label.setObjectName(u"results_label")
        sizePolicy4 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.results_label.sizePolicy().hasHeightForWidth())
        self.results_label.setSizePolicy(sizePolicy4)
        font = QFont()
        font.setBold(True)
        self.results_label.setFont(font)

        self.results_layout.addWidget(self.results_label)

        self.search_results_tree_view = QTreeView(self.layoutWidget_2)
        self.search_results_tree_view.setObjectName(u"search_results_tree_view")
        sizePolicy5 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy5.setHorizontalStretch(0)
        sizePolicy5.setVerticalStretch(0)
        sizePolicy5.setHeightForWidth(self.search_results_tree_view.sizePolicy().hasHeightForWidth())
        self.search_results_tree_view.setSizePolicy(sizePolicy5)
        self.search_results_tree_view.setFrameShape(QFrame.Shape.NoFrame)
        self.search_results_tree_view.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustIgnored)
        self.search_results_tree_view.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
        self.search_results_tree_view.setSortingEnabled(True)
        self.search_results_tree_view.setAnimated(True)
        self.search_results_tree_view.setAllColumnsShowFocus(True)

        self.results_layout.addWidget(self.search_results_tree_view)

        self.splitter.addWidget(self.layoutWidget_2)
        self.layoutWidget_3 = QWidget(self.splitter)
        self.layoutWidget_3.setObjectName(u"layoutWidget_3")
        self.pkg_info_layout = QVBoxLayout(self.layoutWidget_3)
        self.pkg_info_layout.setObjectName(u"pkg_info_layout")
        self.pkg_info_layout.setSizeConstraint(QLayout.SizeConstraint.SetMinAndMaxSize)
        self.pkg_info_layout.setContentsMargins(4, 0, 0, 0)
        self.package_info_label = QLabel(self.layoutWidget_3)
        self.package_info_label.setObjectName(u"package_info_label")
        sizePolicy4.setHeightForWidth(self.package_info_label.sizePolicy().hasHeightForWidth())
        self.package_info_label.setSizePolicy(sizePolicy4)
        self.package_info_label.setMinimumSize(QSize(20, 0))
        self.package_info_label.setBaseSize(QSize(20, 0))
        self.package_info_label.setFont(font)

        self.pkg_info_layout.addWidget(self.package_info_label)

        self.package_info_text = QTextBrowser(self.layoutWidget_3)
        self.package_info_text.setObjectName(u"package_info_text")
        sizePolicy5.setHeightForWidth(self.package_info_text.sizePolicy().hasHeightForWidth())
        self.package_info_text.setSizePolicy(sizePolicy5)
        self.package_info_text.setMinimumSize(QSize(200, 20))
        self.package_info_text.setFrameShape(QFrame.Shape.NoFrame)
        self.package_info_text.setSizeAdjustPolicy(QAbstractScrollArea.SizeAdjustPolicy.AdjustToContents)

        self.pkg_info_layout.addWidget(self.package_info_text)

        self.splitter.addWidget(self.layoutWidget_3)

        self.verticalLayout.addWidget(self.splitter)

        QWidget.setTabOrder(self.search_line, self.search_button)
        QWidget.setTabOrder(self.search_button, self.install_button)
        QWidget.setTabOrder(self.install_button, self.remote_toggle_button)
        QWidget.setTabOrder(self.remote_toggle_button, self.search_results_tree_view)
        QWidget.setTabOrder(self.search_results_tree_view, self.package_info_text)

        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Conan Search", None))
        self.search_line.setText("")
        self.search_line.setPlaceholderText(QCoreApplication.translate("Form", u"*", None))
        self.search_button.setText(QCoreApplication.translate("Form", u"Search", None))
        self.install_button.setText(QCoreApplication.translate("Form", u"Install", None))
        self.remote_toggle_button.setText(QCoreApplication.translate("Form", u"Remotes", None))

        __sortingEnabled = self.select_all_widget.isSortingEnabled()
        self.select_all_widget.setSortingEnabled(False)
        ___qlistwidgetitem = self.select_all_widget.item(0)
        ___qlistwidgetitem.setText(QCoreApplication.translate("Form", u"Select/Deselect All", None));
        self.select_all_widget.setSortingEnabled(__sortingEnabled)

        self.results_label.setText(QCoreApplication.translate("Form", u"Results", None))
        self.package_info_label.setText(QCoreApplication.translate("Form", u"Package Info", None))
    # retranslateUi





from pathlib import Path
from typing import List, Optional

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QListWidgetItem, QWidget

import conan_explorer.app as app
from conan_explorer.app import LoaderGui
from conan_explorer.conan_wrapper.types import Remote  # using global module pattern
from conan_explorer.ui.common.theming import get_themed_asset_icon
from conan_explorer.ui.views.conan_conf.remotes_controller import \
    ConanRemoteController

current_dir = Path(__file__).parent


class RemoteLoginDialog(QDialog):
    """
    Dialog mask for login remotes. 
    The password will be stored by Qt obfuscated, but reversible in RAM.
    To fully lock it out, a manual C++ impl. of QLineEdit would be needed.
    After the dialog is closed, it will be overwritten by a long string.
    """

    def __init__(self, remotes: List[Remote], remotes_controller: ConanRemoteController, parent: Optional[QWidget]):
        super().__init__(parent=parent)
        self._remotes = remotes
        self._remotes_controller = remotes_controller
        from .remote_login_dialog_ui import Ui_Dialog
        self._ui = Ui_Dialog()
        self._ui.setupUi(self)

        # set username, form first name match
        for remote in remotes:
            (name, _) = app.conan_api.get_remote_user_info(remote.name)
            if name:
                self._ui.name_line_edit.setText(name)
                break
        # fill up remote checkbox list
        for remote in remotes:
            item = QListWidgetItem(remote.name, self._ui.remote_list)
            item.setFlags(item.flags() | Qt.ItemFlag.ItemIsUserCheckable)
            item.setCheckState(Qt.CheckState.Checked)
        self.setWindowIcon(get_themed_asset_icon("icons/login.svg", True))
        self._ui.button_box.accepted.connect(self.on_ok)
        self.adjustSize()

    def on_ok(self):
        """ Is triggered on OK and tries to login while showing a loading dialog """
        loader = LoaderGui(self)
        loader.load_with_finish_hook(
            self, self.login, loading_text="Logging you in...", finish_task=self.accept)

    def login(self):
        """ Try to login to all selected remotes """
        selected_remotes: List[str] = []
        for idx in range(self._ui.remote_list.count()):
            if self._ui.remote_list.item(idx).checkState() == Qt.CheckState.Checked:
                selected_remotes.append(self._ui.remote_list.item(idx).data(0))
        self._remotes_controller.login_remotes(selected_remotes, self._ui.name_line_edit.text(),
                                               self._ui.password_line_edit.text())
        self.clear_password()

    def clear_password(self):
        """ This will override the currently stored password. Setting an empty string 
        or clear will not work, the string will still be in memory! """
        long_string = "A" * len(self._ui.password_line_edit.text())
        self._ui.password_line_edit.setText(long_string)
        self._ui.password_line_edit.setText("")
        # run gc to clear all possible dangling Python strings
        import gc
        gc.collect()



from typing import TYPE_CHECKING, Optional

from PySide6.QtGui import QShowEvent
from PySide6.QtWidgets import QFileDialog, QMessageBox, QWidget

from conan_explorer import AUTHOR, BUILT_IN_PLUGIN, DEBUG_LEVEL, base_path
from conan_explorer.ui.plugin import PluginDescription, PluginHandler, PluginInterfaceV1
from conan_explorer.ui.views.plugins_manager.model import PluginModelItem

from .controller import PluginController

if TYPE_CHECKING:
    from conan_explorer.ui.fluent_window import FluentWindow
    from conan_explorer.ui.main_window import BaseSignals


class PluginsPage(PluginInterfaceV1):
    """GUI representation of plugin_handler"""

    def __init__(
        self,
        parent: QWidget,
        plugin_handler: PluginHandler,
        base_signals: Optional["BaseSignals"] = None,
        page_widgets: Optional["FluentWindow.PageStore"] = None,
    ):
        plugin_descr = PluginDescription(
            "Plugin Manager", BUILT_IN_PLUGIN, AUTHOR, "", "", "", " ", False, ""
        )
        super().__init__(parent, plugin_descr)
        from .plugins_ui import Ui_Form

        self._ui = Ui_Form()
        self._ui.setupUi(self)
        self.setObjectName("plugin_manager")
        self._controller = PluginController(self._ui.plugins_tree_view, plugin_handler)
        self._controller.update()
        self.set_themed_icon(self._ui.add_plugin_button, "icons/plus_rounded.svg")
        self.set_themed_icon(self._ui.remove_plugin_button, "icons/delete.svg")
        self.set_themed_icon(self._ui.reload_plugin_button, "icons/refresh.svg")

        # self.set_themed_icon(self._ui.search_icon, "icons/search_packages.svg", size=(20,20))
        self._ui.plugins_tree_view.selectionModel().selectionChanged.connect(
            self.on_plugin_selected
        )
        self._ui.add_plugin_button.clicked.connect(self.on_add)
        self._ui.remove_plugin_button.clicked.connect(self.on_remove)
        self._ui.reload_plugin_button.clicked.connect(self.on_reload)

        if DEBUG_LEVEL < 1:  # only in dev mode
            self._ui.reload_plugin_button.hide()

    def showEvent(self, a0: QShowEvent) -> None:
        self.resize_file_columns()
        return super().showEvent(a0)

    def resize_file_columns(self):
        if not self._ui.plugins_tree_view:
            return
        for i in reversed(range(5 - 1)):
            self._ui.plugins_tree_view.resizeColumnToContents(i)

    def on_plugin_selected(self):
        """Show path of plugin and disable remove for builtins"""
        plugin = self._controller.get_selected_source_item()
        if not plugin:
            return
        self._ui.path_label.setText(plugin.plugin_path)
        if plugin.data(1) == BUILT_IN_PLUGIN:
            self.set_themed_icon(
                self._ui.remove_plugin_button, "icons/delete.svg", force_light_mode=True
            )
            self._ui.remove_plugin_button.setEnabled(False)
        else:
            self.set_themed_icon(self._ui.remove_plugin_button, "icons/delete.svg")
            self._ui.remove_plugin_button.setEnabled(True)

    def on_add(self):
        """Open File dialog with filter for ini files, then load the plugin"""
        dialog = QFileDialog(
            parent=self,
            caption="Select Plugin description file",
            filter="Plugin files (*.ini)",
            directory=str(base_path),
        )
        dialog.setFileMode(QFileDialog.FileMode.ExistingFile)
        if dialog.exec() == QFileDialog.DialogCode.Accepted:
            new_file = dialog.selectedFiles()[0]
            self._controller.add_plugin(new_file)
        self._controller.update()

    def on_remove(self):
        """Unload plugin and deregister from ini"""
        selected_item = self._controller.get_selected_source_item()
        if not selected_item:
            return
        if selected_item.data(1) == BUILT_IN_PLUGIN:
            return
        message_box = QMessageBox(parent=self)
        message_box.setWindowTitle("Remove plugin?")
        message_box.setText(
            f"Are you sure, you want to remove the plugin {selected_item.data(0)}?"
        )
        message_box.setStandardButtons(
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        message_box.setIcon(QMessageBox.Icon.Question)
        reply = message_box.exec()
        if reply == QMessageBox.StandardButton.Yes:
            self._controller.remove_plugin(selected_item.plugin_path)
            self._controller.update()

    def on_reload(self):
        selected_item: Optional[PluginModelItem] = self._controller.get_selected_source_item()
        if not selected_item:
            return
        self._controller.reload_plugin(selected_item.data(0))



from typing import Optional, Union

from PySide6.QtCore import QItemSelectionModel, QModelIndex, QPersistentModelIndex
from PySide6.QtWidgets import QAbstractItemView, QDialog, QListView, QTreeView, QWidget
from typing_extensions import Protocol

from conan_explorer.app.logger import Logger
from conan_explorer.ui.common.theming import get_themed_asset_icon


class ReorderingModel(Protocol):
    def index(
        self, row: int, column: int, parent: "QModelIndex | QPersistentModelIndex"
    ) -> QModelIndex: ...

    def columnCount(self, parent: "QModelIndex | QPersistentModelIndex") -> int: ...

    def moveRow(
        self,
        source_parent: QModelIndex,
        source_row: int,
        destination_parent: QModelIndex,
        destination_child: int,
    ) -> bool: ...

    def beginMoveRows(
        self,
        sourceParent: QModelIndex,
        sourceFirst: int,
        sourceLast: int,
        destinationParent: "QModelIndex | QPersistentModelIndex",
        destinationRow: int,
    ) -> bool: ...

    def endMoveRows(self): ...

    def save(self): ...

    def rowCount(self, parent: Union[QModelIndex, QPersistentModelIndex] = ...) -> int: ...

    ...


class ReorderDialog(QDialog):
    def __init__(self, model: ReorderingModel, parent: Optional[QWidget]):
        super().__init__(parent=parent)
        from .reorder_dialog_ui import Ui_rearrange_dialog

        self._ui = Ui_rearrange_dialog()
        self._ui.setupUi(self)

        self.setWindowIcon(get_themed_asset_icon("icons/rearrange.svg", True))

        self._controller = ReorderController(self._ui.list_view, model)
        self._ui.list_view.setModel(model)  # type: ignore

        self._ui.list_view.setUpdatesEnabled(True)
        self._ui.list_view.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
        self._ui.list_view.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)

        self._ui.move_up_button.clicked.connect(self._controller.move_up)
        self._ui.move_down_button.clicked.connect(self._controller.move_down)
        self._ui.button_box.accepted.connect(self.on_save)

    def on_save(self):
        self._controller.save()
        self.accept()


class ReorderController:
    def __init__(self, view: "QListView| QTreeView", model: ReorderingModel) -> None:
        self._view = view
        self._model = model

    def move_to_position(self, position: int):
        """Moves the selected item up in the list"""
        sel_indexes = self._view.selectedIndexes()
        rows_selected = set([index.row() for index in sel_indexes])

        if len(rows_selected) != 1:
            Logger().info("Select at exactly one item from list!")
            return

        target_position = position
        if position == -1:  # use -1 for last element
            position = self._model.rowCount() - 1
            target_position = position + 1  # insert after last element

        try:
            self._view.selectionModel().clearSelection()
            idx = sel_indexes[0]
            if idx is None:
                return
            row = idx.row()
            if row == position:
                return
            pre_idx = self._model.index(position, 0, self._view.rootIndex())
            self._view.model().beginMoveRows(idx, row, row, pre_idx, pre_idx.row())
            self._view.model().moveRow(idx, row, pre_idx, target_position)
            self._view.model().endMoveRows()
            # for all columns
            for column in range(self._model.columnCount(QModelIndex())):
                index = self._model.index(position, column, QModelIndex())
                self._view.selectionModel().select(
                    index, QItemSelectionModel.SelectionFlag.Select
                )
        except Exception as e:
            print(e)

    def move_up(self):
        """Moves the selected item(s) up in the list"""
        sel_indexes = self._view.selectedIndexes()
        if len(sel_indexes) == 0:
            Logger().info("Select at least one item from list!")
            return

        try:
            # selected indexes need to be sorted
            sel_indexes.sort()  # type: ignore
            first_row = sel_indexes[0].row() - 1
            if first_row < 0:
                return
            self._view.selectionModel().clearSelection()

            rows_moved = []  # optimize for multiple coloumns
            for idx in sel_indexes:
                if idx is None:
                    continue
                row = idx.row()
                if row in rows_moved:
                    continue
                rows_moved.append(row)

                pre_idx = self._model.index(row - 1, 0, self._view.rootIndex())
                self._view.model().beginMoveRows(idx, row, row, pre_idx, pre_idx.row())
                self._view.model().moveRow(idx, row, pre_idx, pre_idx.row())
                self._view.model().endMoveRows()
                # for all columns
                for column in range(self._model.columnCount(QModelIndex())):
                    index = self._model.index(row - 1, column, QModelIndex())
                    self._view.selectionModel().select(
                        index, QItemSelectionModel.SelectionFlag.Select
                    )
        except Exception as e:
            print(e)

    def move_down(self):
        """Moves the selected item(s) down in the list"""
        max_row = self._view.model().rowCount()
        indexes = self._view.selectedIndexes()
        if len(indexes) == 0:
            Logger().info("Select at least one item from list!")
            return

        try:  # modify from reverse so qt index does not get reset
            indexes.sort()  # type: ignore
            self._view.selectionModel().clearSelection()
            last_sel_row = indexes[-1].row() + 1
            if last_sel_row >= max_row:  # cannot be moved down
                return
            rows_moved = []  # optimize for multiple coloumns
            for idx in reversed(indexes):
                if idx is None:
                    continue
                row = idx.row()
                if row in rows_moved:
                    continue
                rows_moved.append(row)

                post_idx = self._model.index(row + 2, 0, self._view.rootIndex())
                post_sel_idx = self._model.index(row + 1, 0, self._view.rootIndex())
                self._model.beginMoveRows(idx, row, row, post_sel_idx, post_sel_idx.row())
                self._model.moveRow(idx, row, post_idx, row + 2)
                self._model.endMoveRows()

                # for all columns
                for column in range(self._model.columnCount(QModelIndex())):
                    index = self._model.index(row + 1, column, QModelIndex())
                    self._view.selectionModel().select(
                        index, QItemSelectionModel.SelectionFlag.Select
                    )
        except Exception as e:
            print(e)

    def save(self):
        self._model.save()



from typing import Literal, Union

from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QColor, QFont, QSyntaxHighlighter, QTextCharFormat

import conan_explorer.app as app
from conan_explorer.settings import GUI_MODE, GUI_MODE_DARK


class ConfigHighlighter(QSyntaxHighlighter):
    """
    Syntax highlighting for Conan Cinfig files: ini and yaml.
    Support dark and light mode.
    """

    def __init__(self, parent, type: Union[Literal["ini"], Literal["yaml"]]) -> None:
        super().__init__(parent)
        self._type = type
        if app.active_settings.get(GUI_MODE) == GUI_MODE_DARK:
            self.key_color = "#9BDCFE"
            self.value_color = "#CE9178"
            self.section_color = "#50C9B1"
        else:
            self.section_color = "#257F99"
            self.key_color = "#001EFD"
            self.value_color = "#AC1613"
        self.comment_color = "#6B9857"

        # key value constants
        if type == "ini":
            self._separator = "="
            self._key_regex = r"(.*?)(?=\=)"
            self._value_regex = r"(?<=\=)(.*?)$"
        elif type == "yaml":
            self._separator = ":"
            self._key_regex = "(.*?)(?=:)"
            self._value_regex = "(?<=:)(.*?)$"

    def highlightBlock(self, text: str):
        # set default value color for everything
        # modify every other construct with special cases
        value_format = QTextCharFormat()
        value_format.setForeground(QColor(self.value_color))
        if self._separator not in text:
            self.setFormat(0, len(text), value_format)
        else:
            expression = QRegularExpression(self._value_regex)
            match = expression.match(text)
            self.setFormat(match.capturedStart(), match.capturedLength(), value_format)

        # sections
        if self._type == "ini":
            section_format = QTextCharFormat()
            section_format.setFontWeight(QFont.Weight.Bold)
            section_format.setForeground(QColor(self.section_color))
            expression = QRegularExpression(r"\[(.*?)\]")
            i = expression.globalMatch(text)
            while i.hasNext():
                match = i.next()
                self.setFormat(match.capturedStart(), match.capturedLength(), section_format)
        # key-values
        key_format = QTextCharFormat()
        key_format.setForeground(QColor(self.key_color))
        expression = QRegularExpression(self._key_regex)
        match = expression.match(text)
        self.setFormat(match.capturedStart(), match.capturedLength(), key_format)

        # comments
        comment_format = QTextCharFormat()
        comment_format.setForeground(QColor(self.comment_color))
        expression = QRegularExpression(r"#(?:\s.*?)$")
        i = expression.globalMatch(text)
        while i.hasNext():
            match = i.next()
            self.setFormat(match.capturedStart(), match.capturedLength(), comment_format)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'conan_conf.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractItemView, QApplication, QFrame, QGridLayout,
    QGroupBox, QHBoxLayout, QHeaderView, QLabel,
    QListView, QPushButton, QScrollArea, QSizePolicy,
    QSpacerItem, QSplitter, QTabWidget, QTreeView,
    QVBoxLayout, QWidget)

from conan_explorer.ui.widgets import (AnimatedToggle, PlainTextPasteBrowser)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(495, 464)
        Form.setStyleSheet(u"")
        self.verticalLayout = QVBoxLayout(Form)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(1, 2, 2, 2)
        self.config_tab_widget = QTabWidget(Form)
        self.config_tab_widget.setObjectName(u"config_tab_widget")
        self.config_tab_widget.setStyleSheet(u"QPushButton {Text-align:left}")
        self.info_widget = QWidget()
        self.info_widget.setObjectName(u"info_widget")
        self.info_widget.setStyleSheet(u"")
        self.verticalLayout_2 = QVBoxLayout(self.info_widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.info_scroll_area = QScrollArea(self.info_widget)
        self.info_scroll_area.setObjectName(u"info_scroll_area")
        self.info_scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        self.info_scroll_area.setWidgetResizable(True)
        self.info_contents = QWidget()
        self.info_contents.setObjectName(u"info_contents")
        self.info_contents.setGeometry(QRect(0, 0, 486, 431))
        self.verticalLayout_7 = QVBoxLayout(self.info_contents)
        self.verticalLayout_7.setObjectName(u"verticalLayout_7")
        self.versions_box = QGroupBox(self.info_contents)
        self.versions_box.setObjectName(u"versions_box")
        self.gridLayout = QGridLayout(self.versions_box)
        self.gridLayout.setObjectName(u"gridLayout")
        self.python_exe_label = QLabel(self.versions_box)
        self.python_exe_label.setObjectName(u"python_exe_label")

        self.gridLayout.addWidget(self.python_exe_label, 2, 0, 1, 1)

        self.python_exe_value_label = QLabel(self.versions_box)
        self.python_exe_value_label.setObjectName(u"python_exe_value_label")
        self.python_exe_value_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.gridLayout.addWidget(self.python_exe_value_label, 2, 1, 1, 1)

        self.conan_cur_version_label = QLabel(self.versions_box)
        self.conan_cur_version_label.setObjectName(u"conan_cur_version_label")

        self.gridLayout.addWidget(self.conan_cur_version_label, 0, 0, 1, 1)

        self.python_cur_version_value_label = QLabel(self.versions_box)
        self.python_cur_version_value_label.setObjectName(u"python_cur_version_value_label")

        self.gridLayout.addWidget(self.python_cur_version_value_label, 1, 1, 1, 1)

        self.conan_cur_version_value_label = QLabel(self.versions_box)
        self.conan_cur_version_value_label.setObjectName(u"conan_cur_version_value_label")
        self.conan_cur_version_value_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.gridLayout.addWidget(self.conan_cur_version_value_label, 0, 1, 1, 1)

        self.python_cur_version_label = QLabel(self.versions_box)
        self.python_cur_version_label.setObjectName(u"python_cur_version_label")

        self.gridLayout.addWidget(self.python_cur_version_label, 1, 0, 1, 1)

        self.gridLayout.setColumnStretch(0, 1)
        self.gridLayout.setColumnStretch(1, 2)

        self.verticalLayout_7.addWidget(self.versions_box)

        self.paths_box = QGroupBox(self.info_contents)
        self.paths_box.setObjectName(u"paths_box")
        self.gridLayout_2 = QGridLayout(self.paths_box)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.conan_usr_home_label = QLabel(self.paths_box)
        self.conan_usr_home_label.setObjectName(u"conan_usr_home_label")

        self.gridLayout_2.addWidget(self.conan_usr_home_label, 0, 0, 1, 1)

        self.conan_usr_home_value_label = QLabel(self.paths_box)
        self.conan_usr_home_value_label.setObjectName(u"conan_usr_home_value_label")
        self.conan_usr_home_value_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.gridLayout_2.addWidget(self.conan_usr_home_value_label, 0, 1, 1, 1)

        self.conan_storage_path_label = QLabel(self.paths_box)
        self.conan_storage_path_label.setObjectName(u"conan_storage_path_label")

        self.gridLayout_2.addWidget(self.conan_storage_path_label, 1, 0, 1, 1)

        self.conan_storage_path_value_label = QLabel(self.paths_box)
        self.conan_storage_path_value_label.setObjectName(u"conan_storage_path_value_label")
        self.conan_storage_path_value_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.gridLayout_2.addWidget(self.conan_storage_path_value_label, 1, 1, 1, 1)

        self.conan_usr_cache_label = QLabel(self.paths_box)
        self.conan_usr_cache_label.setObjectName(u"conan_usr_cache_label")

        self.gridLayout_2.addWidget(self.conan_usr_cache_label, 2, 0, 1, 1)

        self.conan_usr_cache_value_label = QLabel(self.paths_box)
        self.conan_usr_cache_value_label.setObjectName(u"conan_usr_cache_value_label")
        self.conan_usr_cache_value_label.setTextInteractionFlags(Qt.TextInteractionFlag.LinksAccessibleByMouse|Qt.TextInteractionFlag.TextSelectableByKeyboard|Qt.TextInteractionFlag.TextSelectableByMouse)

        self.gridLayout_2.addWidget(self.conan_usr_cache_value_label, 2, 1, 1, 1)

        self.gridLayout_2.setColumnStretch(0, 1)
        self.gridLayout_2.setColumnStretch(1, 2)

        self.verticalLayout_7.addWidget(self.paths_box)

        self.features_box = QGroupBox(self.info_contents)
        self.features_box.setObjectName(u"features_box")
        self.features_box.setFlat(False)
        self.features_box.setCheckable(False)
        self.gridLayout_3 = QGridLayout(self.features_box)
        self.gridLayout_3.setObjectName(u"gridLayout_3")
        self.revision_enabled_label = QLabel(self.features_box)
        self.revision_enabled_label.setObjectName(u"revision_enabled_label")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.revision_enabled_label.sizePolicy().hasHeightForWidth())
        self.revision_enabled_label.setSizePolicy(sizePolicy)
        self.revision_enabled_label.setMaximumSize(QSize(16777215, 16777215))
        self.revision_enabled_label.setMidLineWidth(0)

        self.gridLayout_3.addWidget(self.revision_enabled_label, 0, 0, 1, 1)

        self.revision_enabled_checkbox = AnimatedToggle(self.features_box)
        self.revision_enabled_checkbox.setObjectName(u"revision_enabled_checkbox")
        self.revision_enabled_checkbox.setEnabled(False)
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.revision_enabled_checkbox.sizePolicy().hasHeightForWidth())
        self.revision_enabled_checkbox.setSizePolicy(sizePolicy1)
        self.revision_enabled_checkbox.setCheckable(True)

        self.gridLayout_3.addWidget(self.revision_enabled_checkbox, 0, 1, 1, 1)

        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.gridLayout_3.addItem(self.horizontalSpacer, 0, 2, 1, 1)

        self.gridLayout_3.setColumnStretch(0, 1)
        self.gridLayout_3.setColumnStretch(1, 1)
        self.gridLayout_3.setColumnStretch(2, 2)

        self.verticalLayout_7.addWidget(self.features_box)

        self.verticalSpacer = QSpacerItem(20, 800, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_7.addItem(self.verticalSpacer)

        self.info_scroll_area.setWidget(self.info_contents)

        self.verticalLayout_2.addWidget(self.info_scroll_area)

        self.config_tab_widget.addTab(self.info_widget, "")
        self.remotes = QWidget()
        self.remotes.setObjectName(u"remotes")
        self.verticalLayout_4 = QVBoxLayout(self.remotes)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.remotes_frame = QFrame(self.remotes)
        self.remotes_frame.setObjectName(u"remotes_frame")
        self.remotes_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.remotes_frame.setFrameShadow(QFrame.Shadow.Raised)
        self.gridLayout_4 = QGridLayout(self.remotes_frame)
        self.gridLayout_4.setObjectName(u"gridLayout_4")
        self.gridLayout_4.setContentsMargins(0, -1, 0, -1)
        self.remotes_buttons_frame = QFrame(self.remotes_frame)
        self.remotes_buttons_frame.setObjectName(u"remotes_buttons_frame")
        self.remotes_buttons_frame.setStyleSheet(u"QPushButton {Text-align:left}")
        self.remotes_buttons_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.remotes_buttons_frame.setFrameShadow(QFrame.Shadow.Raised)
        self.verticalLayout_8 = QVBoxLayout(self.remotes_buttons_frame)
        self.verticalLayout_8.setObjectName(u"verticalLayout_8")
        self.verticalLayout_8.setContentsMargins(0, -1, 0, -1)
        self.remote_refresh_button = QPushButton(self.remotes_buttons_frame)
        self.remote_refresh_button.setObjectName(u"remote_refresh_button")

        self.verticalLayout_8.addWidget(self.remote_refresh_button)

        self.remote_move_top_button = QPushButton(self.remotes_buttons_frame)
        self.remote_move_top_button.setObjectName(u"remote_move_top_button")

        self.verticalLayout_8.addWidget(self.remote_move_top_button)

        self.remote_move_up_button = QPushButton(self.remotes_buttons_frame)
        self.remote_move_up_button.setObjectName(u"remote_move_up_button")

        self.verticalLayout_8.addWidget(self.remote_move_up_button)

        self.remote_move_down_button = QPushButton(self.remotes_buttons_frame)
        self.remote_move_down_button.setObjectName(u"remote_move_down_button")

        self.verticalLayout_8.addWidget(self.remote_move_down_button)

        self.remote_move_bottom_button = QPushButton(self.remotes_buttons_frame)
        self.remote_move_bottom_button.setObjectName(u"remote_move_bottom_button")

        self.verticalLayout_8.addWidget(self.remote_move_bottom_button)

        self.remote_login_button = QPushButton(self.remotes_buttons_frame)
        self.remote_login_button.setObjectName(u"remote_login_button")

        self.verticalLayout_8.addWidget(self.remote_login_button)

        self.remote_toggle_disabled_button = QPushButton(self.remotes_buttons_frame)
        self.remote_toggle_disabled_button.setObjectName(u"remote_toggle_disabled_button")

        self.verticalLayout_8.addWidget(self.remote_toggle_disabled_button)

        self.remote_add_button = QPushButton(self.remotes_buttons_frame)
        self.remote_add_button.setObjectName(u"remote_add_button")

        self.verticalLayout_8.addWidget(self.remote_add_button)

        self.remote_remove_button = QPushButton(self.remotes_buttons_frame)
        self.remote_remove_button.setObjectName(u"remote_remove_button")

        self.verticalLayout_8.addWidget(self.remote_remove_button)

        self.remotes_edit_button = QPushButton(self.remotes_buttons_frame)
        self.remotes_edit_button.setObjectName(u"remotes_edit_button")

        self.verticalLayout_8.addWidget(self.remotes_edit_button)

        self.remotes_btns_spacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_8.addItem(self.remotes_btns_spacer)


        self.gridLayout_4.addWidget(self.remotes_buttons_frame, 0, 1, 1, 1)

        self.remotes_tree_view = QTreeView(self.remotes_frame)
        self.remotes_tree_view.setObjectName(u"remotes_tree_view")
        self.remotes_tree_view.setFrameShape(QFrame.Shape.NoFrame)
        self.remotes_tree_view.setDragEnabled(False)
        self.remotes_tree_view.setSelectionMode(QAbstractItemView.SelectionMode.ExtendedSelection)
        self.remotes_tree_view.setSelectionBehavior(QAbstractItemView.SelectionBehavior.SelectRows)
        self.remotes_tree_view.setUniformRowHeights(False)
        self.remotes_tree_view.setSortingEnabled(False)
        self.remotes_tree_view.setAnimated(True)
        self.remotes_tree_view.setAllColumnsShowFocus(True)
        self.remotes_tree_view.setWordWrap(False)
        self.remotes_tree_view.setExpandsOnDoubleClick(False)

        self.gridLayout_4.addWidget(self.remotes_tree_view, 0, 0, 1, 1)


        self.verticalLayout_4.addWidget(self.remotes_frame)

        self.config_tab_widget.addTab(self.remotes, "")
        self.profiles = QWidget()
        self.profiles.setObjectName(u"profiles")
        self.verticalLayout_11 = QVBoxLayout(self.profiles)
        self.verticalLayout_11.setObjectName(u"verticalLayout_11")
        self.verticalLayout_11.setContentsMargins(0, 0, 0, 0)
        self.profiles_grid_layout = QGridLayout()
        self.profiles_grid_layout.setObjectName(u"profiles_grid_layout")
        self.splitter = QSplitter(self.profiles)
        self.splitter.setObjectName(u"splitter")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Expanding)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.splitter.sizePolicy().hasHeightForWidth())
        self.splitter.setSizePolicy(sizePolicy2)
        self.splitter.setOrientation(Qt.Orientation.Vertical)
        self.profiles_list_view = QListView(self.splitter)
        self.profiles_list_view.setObjectName(u"profiles_list_view")
        self.profiles_list_view.setFrameShape(QFrame.Shape.NoFrame)
        self.profiles_list_view.setMovement(QListView.Movement.Free)
        self.profiles_list_view.setUniformItemSizes(True)
        self.splitter.addWidget(self.profiles_list_view)
        self.profiles_text_browser = PlainTextPasteBrowser(self.splitter)
        self.profiles_text_browser.setObjectName(u"profiles_text_browser")
        self.profiles_text_browser.setUndoRedoEnabled(True)
        self.profiles_text_browser.setReadOnly(False)
        self.splitter.addWidget(self.profiles_text_browser)

        self.profiles_grid_layout.addWidget(self.splitter, 0, 0, 1, 1)

        self.profiles_buttons_frame = QFrame(self.profiles)
        self.profiles_buttons_frame.setObjectName(u"profiles_buttons_frame")
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.profiles_buttons_frame.sizePolicy().hasHeightForWidth())
        self.profiles_buttons_frame.setSizePolicy(sizePolicy3)
        self.profiles_buttons_frame.setStyleSheet(u"QPushButton {Text-align:left}")
        self.profiles_buttons_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.profiles_buttons_frame.setFrameShadow(QFrame.Shadow.Raised)
        self.verticalLayout_9 = QVBoxLayout(self.profiles_buttons_frame)
        self.verticalLayout_9.setObjectName(u"verticalLayout_9")
        self.verticalLayout_9.setContentsMargins(0, -1, 0, -1)
        self.profile_refresh_button = QPushButton(self.profiles_buttons_frame)
        self.profile_refresh_button.setObjectName(u"profile_refresh_button")

        self.verticalLayout_9.addWidget(self.profile_refresh_button)

        self.profile_add_button = QPushButton(self.profiles_buttons_frame)
        self.profile_add_button.setObjectName(u"profile_add_button")

        self.verticalLayout_9.addWidget(self.profile_add_button)

        self.profile_remove_button = QPushButton(self.profiles_buttons_frame)
        self.profile_remove_button.setObjectName(u"profile_remove_button")

        self.verticalLayout_9.addWidget(self.profile_remove_button)

        self.profile_rename_button = QPushButton(self.profiles_buttons_frame)
        self.profile_rename_button.setObjectName(u"profile_rename_button")

        self.verticalLayout_9.addWidget(self.profile_rename_button)

        self.profile_save_button = QPushButton(self.profiles_buttons_frame)
        self.profile_save_button.setObjectName(u"profile_save_button")

        self.verticalLayout_9.addWidget(self.profile_save_button)

        self.profiles_copy_name_button = QPushButton(self.profiles_buttons_frame)
        self.profiles_copy_name_button.setObjectName(u"profiles_copy_name_button")

        self.verticalLayout_9.addWidget(self.profiles_copy_name_button)

        self.profiles_btns_spacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_9.addItem(self.profiles_btns_spacer)


        self.profiles_grid_layout.addWidget(self.profiles_buttons_frame, 0, 1, 1, 1)


        self.verticalLayout_11.addLayout(self.profiles_grid_layout)

        self.config_tab_widget.addTab(self.profiles, "")
        self.editables = QWidget()
        self.editables.setObjectName(u"editables")
        self.verticalLayout_5 = QVBoxLayout(self.editables)
        self.verticalLayout_5.setObjectName(u"verticalLayout_5")
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.editables_grid_layout = QGridLayout()
        self.editables_grid_layout.setObjectName(u"editables_grid_layout")
        self.editables_splitter = QSplitter(self.editables)
        self.editables_splitter.setObjectName(u"editables_splitter")
        sizePolicy2.setHeightForWidth(self.editables_splitter.sizePolicy().hasHeightForWidth())
        self.editables_splitter.setSizePolicy(sizePolicy2)
        self.editables_splitter.setOrientation(Qt.Orientation.Vertical)
        self.editables_ref_view = QTreeView(self.editables_splitter)
        self.editables_ref_view.setObjectName(u"editables_ref_view")
        self.editables_ref_view.setFrameShape(QFrame.Shape.NoFrame)
        self.editables_splitter.addWidget(self.editables_ref_view)

        self.editables_grid_layout.addWidget(self.editables_splitter, 0, 0, 1, 1)

        self.editables_button_frame = QFrame(self.editables)
        self.editables_button_frame.setObjectName(u"editables_button_frame")
        self.editables_button_frame.setStyleSheet(u"QPushButton {Text-align:left}")
        self.editables_button_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.editables_button_frame.setFrameShadow(QFrame.Shadow.Raised)
        self.verticalLayout_10 = QVBoxLayout(self.editables_button_frame)
        self.verticalLayout_10.setObjectName(u"verticalLayout_10")
        self.verticalLayout_10.setContentsMargins(0, -1, 0, -1)
        self.editables_refresh_button = QPushButton(self.editables_button_frame)
        self.editables_refresh_button.setObjectName(u"editables_refresh_button")

        self.verticalLayout_10.addWidget(self.editables_refresh_button)

        self.editables_add_button = QPushButton(self.editables_button_frame)
        self.editables_add_button.setObjectName(u"editables_add_button")

        self.verticalLayout_10.addWidget(self.editables_add_button)

        self.editables_remove_button = QPushButton(self.editables_button_frame)
        self.editables_remove_button.setObjectName(u"editables_remove_button")

        self.verticalLayout_10.addWidget(self.editables_remove_button)

        self.editables_edit_button = QPushButton(self.editables_button_frame)
        self.editables_edit_button.setObjectName(u"editables_edit_button")

        self.verticalLayout_10.addWidget(self.editables_edit_button)

        self.editables_button_vspacer = QSpacerItem(20, 40, QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Expanding)

        self.verticalLayout_10.addItem(self.editables_button_vspacer)


        self.editables_grid_layout.addWidget(self.editables_button_frame, 0, 1, 1, 1)


        self.verticalLayout_5.addLayout(self.editables_grid_layout)

        self.config_tab_widget.addTab(self.editables, "")
        self.config = QWidget()
        self.config.setObjectName(u"config")
        self.verticalLayout_3 = QVBoxLayout(self.config)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.config_file_text_browser = PlainTextPasteBrowser(self.config)
        self.config_file_text_browser.setObjectName(u"config_file_text_browser")
        self.config_file_text_browser.setTabChangesFocus(False)
        self.config_file_text_browser.setUndoRedoEnabled(True)
        self.config_file_text_browser.setReadOnly(False)

        self.verticalLayout_3.addWidget(self.config_file_text_browser)

        self.config_btns_frame = QFrame(self.config)
        self.config_btns_frame.setObjectName(u"config_btns_frame")
        self.config_btns_frame.setFrameShape(QFrame.Shape.StyledPanel)
        self.config_btns_frame.setFrameShadow(QFrame.Shadow.Raised)
        self.horizontalLayout = QHBoxLayout(self.config_btns_frame)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.config_file_save_button = QPushButton(self.config_btns_frame)
        self.config_file_save_button.setObjectName(u"config_file_save_button")

        self.horizontalLayout.addWidget(self.config_file_save_button)

        self.config_btns_spacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout.addItem(self.config_btns_spacer)


        self.verticalLayout_3.addWidget(self.config_btns_frame)

        self.config_tab_widget.addTab(self.config, "")
        self.settings_file = QWidget()
        self.settings_file.setObjectName(u"settings_file")
        self.verticalLayout_6 = QVBoxLayout(self.settings_file)
        self.verticalLayout_6.setObjectName(u"verticalLayout_6")
        self.verticalLayout_6.setContentsMargins(0, 0, 0, 0)
        self.settings_file_text_browser = PlainTextPasteBrowser(self.settings_file)
        self.settings_file_text_browser.setObjectName(u"settings_file_text_browser")

        self.verticalLayout_6.addWidget(self.settings_file_text_browser)

        self.config_tab_widget.addTab(self.settings_file, "")

        self.verticalLayout.addWidget(self.config_tab_widget)


        self.retranslateUi(Form)

        self.config_tab_widget.setCurrentIndex(1)


        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.versions_box.setTitle(QCoreApplication.translate("Form", u"Versions", None))
        self.python_exe_label.setText(QCoreApplication.translate("Form", u"Current Python Executable:", None))
        self.python_exe_value_label.setText(QCoreApplication.translate("Form", u"Unknown", None))
        self.conan_cur_version_label.setText(QCoreApplication.translate("Form", u"Current Conan Version:", None))
        self.python_cur_version_value_label.setText(QCoreApplication.translate("Form", u"Unknown", None))
        self.conan_cur_version_value_label.setText(QCoreApplication.translate("Form", u"Unknown", None))
        self.python_cur_version_label.setText(QCoreApplication.translate("Form", u"Current Python Version:", None))
        self.paths_box.setTitle(QCoreApplication.translate("Form", u"Paths", None))
        self.conan_usr_home_label.setText(QCoreApplication.translate("Form", u"Conan User Home:", None))
        self.conan_usr_home_value_label.setText("")
        self.conan_storage_path_label.setText(QCoreApplication.translate("Form", u"Conan Storage Path:", None))
        self.conan_storage_path_value_label.setText("")
        self.conan_usr_cache_label.setText(QCoreApplication.translate("Form", u"Conan Short Path Cache:", None))
        self.conan_usr_cache_value_label.setText("")
        self.features_box.setTitle(QCoreApplication.translate("Form", u"Features", None))
        self.revision_enabled_label.setText(QCoreApplication.translate("Form", u"Revisions enabled:", None))
        self.revision_enabled_checkbox.setText("")
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.info_widget), QCoreApplication.translate("Form", u"Info", None))
        self.remote_refresh_button.setText(QCoreApplication.translate("Form", u"Refresh list", None))
        self.remote_move_top_button.setText(QCoreApplication.translate("Form", u"Move to Top", None))
        self.remote_move_up_button.setText(QCoreApplication.translate("Form", u"Move Up", None))
        self.remote_move_down_button.setText(QCoreApplication.translate("Form", u"Move Down", None))
        self.remote_move_bottom_button.setText(QCoreApplication.translate("Form", u"Move to Bottom", None))
        self.remote_login_button.setText(QCoreApplication.translate("Form", u"(Multi)Login", None))
        self.remote_toggle_disabled_button.setText(QCoreApplication.translate("Form", u"Enable/Disable", None))
        self.remote_add_button.setText(QCoreApplication.translate("Form", u"Add", None))
        self.remote_remove_button.setText(QCoreApplication.translate("Form", u"Remove", None))
        self.remotes_edit_button.setText(QCoreApplication.translate("Form", u"Edit", None))
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.remotes), QCoreApplication.translate("Form", u"Remotes", None))
        self.profile_refresh_button.setText(QCoreApplication.translate("Form", u"Refresh List", None))
        self.profile_add_button.setText(QCoreApplication.translate("Form", u"Add", None))
        self.profile_remove_button.setText(QCoreApplication.translate("Form", u"Remove", None))
        self.profile_rename_button.setText(QCoreApplication.translate("Form", u"Rename", None))
        self.profile_save_button.setText(QCoreApplication.translate("Form", u"Save", None))
        self.profiles_copy_name_button.setText(QCoreApplication.translate("Form", u"Copy Name", None))
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.profiles), QCoreApplication.translate("Form", u"Profiles", None))
        self.editables_refresh_button.setText(QCoreApplication.translate("Form", u"Refresh List", None))
        self.editables_add_button.setText(QCoreApplication.translate("Form", u"Add", None))
        self.editables_remove_button.setText(QCoreApplication.translate("Form", u"Remove", None))
        self.editables_edit_button.setText(QCoreApplication.translate("Form", u"Edit", None))
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.editables), QCoreApplication.translate("Form", u"Editables", None))
        self.config_file_save_button.setText(QCoreApplication.translate("Form", u"Save", None))
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.config), QCoreApplication.translate("Form", u"Config File", None))
        self.config_tab_widget.setTabText(self.config_tab_widget.indexOf(self.settings_file), QCoreApplication.translate("Form", u"Settings YML File", None))
    # retranslateUi




# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'app_link.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QFormLayout, QFrame, QHBoxLayout,
    QLabel, QLayout, QPushButton, QSizePolicy,
    QVBoxLayout, QWidget)

from conan_explorer.ui.widgets import (AnimatedToggle, ClickableIcon)

class Ui_Form(object):
    def setupUi(self, Form):
        if not Form.objectName():
            Form.setObjectName(u"Form")
        Form.resize(882, 140)
        sizePolicy = QSizePolicy(QSizePolicy.Policy.MinimumExpanding, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Form.sizePolicy().hasHeightForWidth())
        Form.setSizePolicy(sizePolicy)
        Form.setMinimumSize(QSize(0, 140))
        self.horizontalLayout_3 = QHBoxLayout(Form)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalLayout_3.setContentsMargins(0, 2, 0, 2)
        self.left_frame = QFrame(Form)
        self.left_frame.setObjectName(u"left_frame")
        self.left_frame.setMinimumSize(QSize(0, 0))
        self.left_frame.setFrameShape(QFrame.StyledPanel)
        self.left_frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout_2 = QVBoxLayout(self.left_frame)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setSizeConstraint(QLayout.SetMinAndMaxSize)
        self.verticalLayout_2.setContentsMargins(-1, 6, -1, -1)
        self.app_button = ClickableIcon(self.left_frame)
        self.app_button.setObjectName(u"app_button")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Fixed)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.app_button.sizePolicy().hasHeightForWidth())
        self.app_button.setSizePolicy(sizePolicy1)
        self.app_button.setMinimumSize(QSize(150, 100))
        self.app_button.setMaximumSize(QSize(150, 100))

        self.verticalLayout_2.addWidget(self.app_button)

        self.app_name_label = QLabel(self.left_frame)
        self.app_name_label.setObjectName(u"app_name_label")
        sizePolicy2 = QSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        sizePolicy2.setHorizontalStretch(0)
        sizePolicy2.setVerticalStretch(0)
        sizePolicy2.setHeightForWidth(self.app_name_label.sizePolicy().hasHeightForWidth())
        self.app_name_label.setSizePolicy(sizePolicy2)
        self.app_name_label.setMinimumSize(QSize(0, 0))
        self.app_name_label.setMaximumSize(QSize(180, 16777215))
        self.app_name_label.setAlignment(Qt.AlignCenter)
        self.app_name_label.setWordWrap(True)

        self.verticalLayout_2.addWidget(self.app_name_label)


        self.horizontalLayout_3.addWidget(self.left_frame)

        self.central_frame = QFrame(Form)
        self.central_frame.setObjectName(u"central_frame")
        sizePolicy3 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        sizePolicy3.setHorizontalStretch(0)
        sizePolicy3.setVerticalStretch(0)
        sizePolicy3.setHeightForWidth(self.central_frame.sizePolicy().hasHeightForWidth())
        self.central_frame.setSizePolicy(sizePolicy3)
        self.central_frame.setMinimumSize(QSize(0, 0))
        self.central_frame.setFrameShape(QFrame.StyledPanel)
        self.central_frame.setFrameShadow(QFrame.Raised)
        self.horizontalLayout_2 = QHBoxLayout(self.central_frame)
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.central_left_frame = QFrame(self.central_frame)
        self.central_left_frame.setObjectName(u"central_left_frame")
        sizePolicy2.setHeightForWidth(self.central_left_frame.sizePolicy().hasHeightForWidth())
        self.central_left_frame.setSizePolicy(sizePolicy2)
        self.central_left_frame.setFrameShape(QFrame.StyledPanel)
        self.central_left_frame.setFrameShadow(QFrame.Raised)
        self.formLayout = QFormLayout(self.central_left_frame)
        self.formLayout.setObjectName(u"formLayout")
        self.formLayout.setContentsMargins(-1, -1, 0, -1)
        self.open_shell_checkbox = AnimatedToggle(self.central_left_frame)
        self.open_shell_checkbox.setObjectName(u"open_shell_checkbox")
        self.open_shell_checkbox.setEnabled(False)
        sizePolicy1.setHeightForWidth(self.open_shell_checkbox.sizePolicy().hasHeightForWidth())
        self.open_shell_checkbox.setSizePolicy(sizePolicy1)
        self.open_shell_checkbox.setMaximumSize(QSize(16777215, 35))
        self.open_shell_checkbox.setStyleSheet(u"margin-top:50%; margin-bottom:50%;margin-left: 10%;")

        self.formLayout.setWidget(2, QFormLayout.FieldRole, self.open_shell_checkbox)

        self.open_shell_label = QLabel(self.central_left_frame)
        self.open_shell_label.setObjectName(u"open_shell_label")
        sizePolicy3.setHeightForWidth(self.open_shell_label.sizePolicy().hasHeightForWidth())
        self.open_shell_label.setSizePolicy(sizePolicy3)

        self.formLayout.setWidget(2, QFormLayout.LabelRole, self.open_shell_label)

        self.executable_name_label = QLabel(self.central_left_frame)
        self.executable_name_label.setObjectName(u"executable_name_label")

        self.formLayout.setWidget(3, QFormLayout.LabelRole, self.executable_name_label)

        self.executable_value_label = QLabel(self.central_left_frame)
        self.executable_value_label.setObjectName(u"executable_value_label")

        self.formLayout.setWidget(3, QFormLayout.FieldRole, self.executable_value_label)

        self.conan_ref_name_label = QLabel(self.central_left_frame)
        self.conan_ref_name_label.setObjectName(u"conan_ref_name_label")

        self.formLayout.setWidget(1, QFormLayout.LabelRole, self.conan_ref_name_label)

        self.conan_ref_value_label = QLabel(self.central_left_frame)
        self.conan_ref_value_label.setObjectName(u"conan_ref_value_label")
        sizePolicy3.setHeightForWidth(self.conan_ref_value_label.sizePolicy().hasHeightForWidth())
        self.conan_ref_value_label.setSizePolicy(sizePolicy3)
        self.conan_ref_value_label.setMaximumSize(QSize(16777215, 16777215))

        self.formLayout.setWidget(1, QFormLayout.FieldRole, self.conan_ref_value_label)


        self.horizontalLayout_2.addWidget(self.central_left_frame)

        self.central_right_frame = QFrame(self.central_frame)
        self.central_right_frame.setObjectName(u"central_right_frame")
        self.central_right_frame.setEnabled(True)
        sizePolicy2.setHeightForWidth(self.central_right_frame.sizePolicy().hasHeightForWidth())
        self.central_right_frame.setSizePolicy(sizePolicy2)
        self.central_right_frame.setMaximumSize(QSize(16777215, 16777215))
        self.central_right_frame.setFrameShape(QFrame.StyledPanel)
        self.central_right_frame.setFrameShadow(QFrame.Raised)
        self.formLayout_2 = QFormLayout(self.central_right_frame)
        self.formLayout_2.setObjectName(u"formLayout_2")
        self.formLayout_2.setSizeConstraint(QLayout.SetMinAndMaxSize)
        self.formLayout_2.setFieldGrowthPolicy(QFormLayout.FieldsStayAtSizeHint)
        self.arguments_name_label = QLabel(self.central_right_frame)
        self.arguments_name_label.setObjectName(u"arguments_name_label")
        sizePolicy2.setHeightForWidth(self.arguments_name_label.sizePolicy().hasHeightForWidth())
        self.arguments_name_label.setSizePolicy(sizePolicy2)
        self.arguments_name_label.setMinimumSize(QSize(0, 0))
        self.arguments_name_label.setMaximumSize(QSize(2000, 16777215))
        self.arguments_name_label.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)

        self.formLayout_2.setWidget(0, QFormLayout.LabelRole, self.arguments_name_label)

        self.arguments_value_label = QLabel(self.central_right_frame)
        self.arguments_value_label.setObjectName(u"arguments_value_label")
        sizePolicy2.setHeightForWidth(self.arguments_value_label.sizePolicy().hasHeightForWidth())
        self.arguments_value_label.setSizePolicy(sizePolicy2)
        self.arguments_value_label.setMinimumSize(QSize(0, 0))
        self.arguments_value_label.setMaximumSize(QSize(100, 16777215))
        self.arguments_value_label.setScaledContents(False)
        self.arguments_value_label.setAlignment(Qt.AlignLeading|Qt.AlignLeft|Qt.AlignTop)
        self.arguments_value_label.setWordWrap(True)

        self.formLayout_2.setWidget(0, QFormLayout.FieldRole, self.arguments_value_label)


        self.horizontalLayout_2.addWidget(self.central_right_frame)

        self.horizontalLayout_2.setStretch(0, 1)
        self.horizontalLayout_2.setStretch(1, 2)

        self.horizontalLayout_3.addWidget(self.central_frame)

        self.right_frame = QFrame(Form)
        self.right_frame.setObjectName(u"right_frame")
        self.right_frame.setMinimumSize(QSize(100, 0))
        self.right_frame.setFrameShape(QFrame.StyledPanel)
        self.right_frame.setFrameShadow(QFrame.Raised)
        self.verticalLayout = QVBoxLayout(self.right_frame)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setSizeConstraint(QLayout.SetMinAndMaxSize)
        self.verticalLayout.setContentsMargins(6, 0, -1, 0)
        self.edit_button = QPushButton(self.right_frame)
        self.edit_button.setObjectName(u"edit_button")
        sizePolicy4 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Fixed)
        sizePolicy4.setHorizontalStretch(0)
        sizePolicy4.setVerticalStretch(0)
        sizePolicy4.setHeightForWidth(self.edit_button.sizePolicy().hasHeightForWidth())
        self.edit_button.setSizePolicy(sizePolicy4)
        self.edit_button.setMinimumSize(QSize(150, 0))
        self.edit_button.setMaximumSize(QSize(150, 16777215))

        self.verticalLayout.addWidget(self.edit_button)

        self.remove_button = QPushButton(self.right_frame)
        self.remove_button.setObjectName(u"remove_button")
        sizePolicy4.setHeightForWidth(self.remove_button.sizePolicy().hasHeightForWidth())
        self.remove_button.setSizePolicy(sizePolicy4)
        self.remove_button.setMinimumSize(QSize(150, 0))
        self.remove_button.setMaximumSize(QSize(150, 16777215))

        self.verticalLayout.addWidget(self.remove_button)


        self.horizontalLayout_3.addWidget(self.right_frame)

        self.horizontalLayout_3.setStretch(1, 1)

        self.retranslateUi(Form)

        QMetaObject.connectSlotsByName(Form)
    # setupUi

    def retranslateUi(self, Form):
        Form.setWindowTitle(QCoreApplication.translate("Form", u"Form", None))
        self.app_button.setText("")
        self.app_name_label.setText(QCoreApplication.translate("Form", u"Name", None))
        self.open_shell_checkbox.setText("")
        self.open_shell_label.setText(QCoreApplication.translate("Form", u"<html><head/><body><p><span style=\" font-weight:700;\">Use terminal:</span></p></body></html>", None))
        self.executable_name_label.setText(QCoreApplication.translate("Form", u"<html><head/><body><p><span style=\" font-weight:700;\">Executable:</span></p></body></html>", None))
        self.executable_value_label.setText(QCoreApplication.translate("Form", u"bin/python.exe", None))
        self.conan_ref_name_label.setText(QCoreApplication.translate("Form", u"<html><head/><body><p><span style=\" font-weight:700;\">Reference:</span></p></body></html>", None))
        self.conan_ref_value_label.setText(QCoreApplication.translate("Form", u"example/9.9.9@user/channel", None))
        self.arguments_name_label.setText(QCoreApplication.translate("Form", u"<html><head/><body><p><span style=\" font-weight:700;\">Arguments:</span></p></body></html>", None))
        self.arguments_value_label.setText(QCoreApplication.translate("Form", u"-f silver_output -d C:Reposfep_interface_silver3exampledemo_simplesilver_example.description -t standard_data -s fepifsilver_default_system", None))
        self.edit_button.setText(QCoreApplication.translate("Form", u"Edit ", None))
        self.remove_button.setText(QCoreApplication.translate("Form", u"Remove ", None))
    # retranslateUi




import configparser
import os
from copy import deepcopy
from pathlib import Path
from typing import Any, Dict, List, Optional

from typing_extensions import override

from conan_explorer import BUILT_IN_PLUGIN, PathLike, base_path
from conan_explorer.app.logger import Logger
from conan_explorer.app.system import get_default_file_editor

from . import (
    AUTO_INSTALL_QUICKLAUNCH_REFS,
    AUTO_OPEN_LAST_VIEW,
    CONSOLE_SPLIT_SIZES,
    DEFAULT_INSTALL_PROFILE,
    FILE_EDITOR_EXECUTABLE,
    FONT_SIZE,
    GENERAL_SECTION_NAME,
    GUI_MODE,
    GUI_MODE_LIGHT,
    GUI_STYLE,
    GUI_STYLE_MATERIAL,
    LAST_CONFIG_FILE,
    LAST_VIEW,
    PLUGINS_SECTION_NAME,
    VIEW_SECTION_NAME,
    WINDOW_SIZE,
    SettingsInterface,
)


def application_settings_spec() -> Dict[str, Dict[str, Any]]:
    return {
        GENERAL_SECTION_NAME: {
            LAST_CONFIG_FILE: "",
            FILE_EDITOR_EXECUTABLE: get_default_file_editor(),
            AUTO_INSTALL_QUICKLAUNCH_REFS: False,
            DEFAULT_INSTALL_PROFILE: "",
        },
        VIEW_SECTION_NAME: {
            FONT_SIZE: 13,
            GUI_STYLE: GUI_STYLE_MATERIAL,
            GUI_MODE: GUI_MODE_LIGHT,
            WINDOW_SIZE: "0,0,800,600",
            CONSOLE_SPLIT_SIZES: "413,126",
            LAST_VIEW: "",
            AUTO_OPEN_LAST_VIEW: True,
        },
        PLUGINS_SECTION_NAME: {BUILT_IN_PLUGIN: str(base_path / "ui" / "plugins.ini")},
    }


class IniSettings(SettingsInterface):
    """
    Settings mechanism with an ini file to use as a storage.
    File and entries are automatically created from the default value of the class.
    User defined keys are now allowed for nodes specified incustom_key_enabled_sections.
    Settings should be accessed via their constant name.
    """

    def __init__(
        self,
        ini_file_path: Optional[PathLike],
        auto_save=True,
        default_values: Dict[str, Dict[str, Any]] = application_settings_spec(),
        custom_key_enabled_sections: List[str] = [PLUGINS_SECTION_NAME],
    ):
        """
        Read config.ini file to load settings.
        Create, if not existing, but the directory must already exist!
        Default path is current working dir / settings.ini
        """
        if not ini_file_path:
            self._ini_file_path = Path().cwd() / "settings.ini"
        else:
            self._ini_file_path = Path(ini_file_path)
        self._auto_save = auto_save
        self._custom_key_enabled_sections = custom_key_enabled_sections
        self._logger = Logger()
        self._parser = configparser.ConfigParser()

        ### default setting values ###
        self._values: Dict[str, Dict[str, Any]] = deepcopy(default_values)

        # create Settings ini file, if not available for first start
        if not self._ini_file_path.is_file():
            self._ini_file_path.open("a").close()
            self._logger.info("Settings: Creating settings ini-file")
            self.save()
        else:
            self._logger.info("Settings: Using %s", self._ini_file_path)

        self._read_ini()

    @override
    def set_auto_save(self, value: bool):
        self._auto_save = value

    @override
    def get_settings_from_node(self, node: str) -> List[str]:
        return list(self._values.get(node, {}).keys())

    @override
    def get(self, name: str) -> "str | int | float | bool":
        """Get a specific setting"""
        value = None
        for section in self._values.values():
            if name in section:
                value = section.get(name)
                break
        if value is None:
            raise LookupError
        return value

    @override
    def get_string(self, name: str) -> str:
        return str(self.get(name))

    @override
    def get_int(self, name: str) -> int:
        return int(self.get(name))

    @override
    def get_float(self, name: str) -> float:
        return float(self.get(name))

    @override
    def get_bool(self, name: str) -> bool:
        return bool(self.get(name))

    @override
    def set(self, name: str, value: "str|int|float|bool|dict"):
        """Set the value of a specific setting.
        Does not write to file, if value is already set."""
        if name in self._values.keys() and isinstance(value, dict):  # dict type setting
            if self._values[name] == value:
                return
            self._values[name].update(value)
        else:
            for section in self._values.keys():
                if name in self._values[section]:
                    if self._values[section][name] == value:
                        return
                    self._values[section][name] = value
                    break
        if self._auto_save:
            self.save()

    @override
    def add(self, name: str, value: "str|int|float|bool", node: Optional[str] = None):
        if node is None:
            node = GENERAL_SECTION_NAME
        if not self._values.get(node):
            self._values[node] = {}
        self._values[node][name] = value
        self.save()

    @override
    def remove(self, name: str):
        for node_name, node in self._values.items():
            if name in node:
                node.pop(name)
                del self._parser[node_name][name]
                break
        self.save()

    @override
    def save(self):
        """Save all user modifiable settings to file."""
        # save all default values
        for section in self._values.keys():
            for setting in self._values[section]:
                self._write_setting(setting, section)

        with self._ini_file_path.open("w", encoding="utf-8") as ini_file:
            self._parser.write(ini_file)

    def _read_ini(self):
        """Read settings ini with configparser."""
        update_needed = False
        try:
            self._parser.read(self._ini_file_path, encoding="utf--8")
            for node in self._parser.sections():
                setting_keys = set(list(self._values.get(node, {}).keys()))
                if node in self._custom_key_enabled_sections:
                    setting_keys = setting_keys.union(set(self._get_section(node).keys()))
                # empty section - this is a user filled dict
                if not self._values.get(node):
                    update_needed |= self._read_dict_setting(node)
                for setting in setting_keys:
                    update_needed |= self._read_setting(setting, node)

        except Exception as e:
            Logger().error(
                (
                    f"Settings: Can't read ini file: {str(e)}",
                    "trying to delete and create a new one...",
                )
            )
            try:
                # let an exeception to the user, file can't be deleted
                os.remove(str(self._ini_file_path))
            except Exception:
                Logger().error(f"Settings: Can't delete ini file: {str(e)}.")

        # write file - to record defaults, if missing
        if not update_needed:
            return
        with self._ini_file_path.open("w", encoding="utf8") as ini_file:
            self._parser.write(ini_file)

    def _get_section(self, node: str) -> configparser.SectionProxy:
        """Get a section from ini, or create it, if it does not exist."""
        if node not in self._parser:
            self._parser.add_section(node)
        if node not in self._values:
            self._values[node] = {}
        return self._parser[node]

    def _read_dict_setting(self, node: str) -> bool:
        """
        Get a dict style setting.
        Dict settings are section itself and are read dynamically.
        """
        section = self._get_section(node)
        update_needed = False
        for name in section.keys():
            update_needed |= self._read_setting(name, node)
        return update_needed

    def _read_setting(self, name: str, node: str) -> bool:
        """Get a setting, which uses the init value to determine the type.
        Returns, if file needs tobe updated
        """
        section = self._get_section(node)
        try:
            default_value = self.get(name)
        except Exception:
            default_value = ""
        if isinstance(default_value, dict):  # no dicts supported directly
            return False

        if name not in section:  # write out
            section[name] = str(default_value)
            return True

        value: "str | int | float | bool | None" = None
        if isinstance(default_value, bool):
            value = section.getboolean(name)
        elif isinstance(default_value, str):
            value = str(section.get(name))
        elif isinstance(default_value, float):
            value = float(section.get(name))
        elif isinstance(default_value, int):
            value = int(section.get(name))
        if value is None:  # dict type, value will be taken as a string
            self._logger.error("Settings: Setting %s to write is unknown", name)
            return False
        if value == "" and default_value:
            value = default_value
        # autosave must be disabled temporarily,
        # otherwise we overwrite the other settings in the file
        auto_save = self._auto_save
        self._auto_save = False
        self._values[node][name] = value
        self._auto_save = auto_save
        return False

    def _write_setting(self, name, node):
        """Helper function to write a setting."""
        value = self.get(name)
        if isinstance(value, dict):
            return  # dicts are read only currently

        section = self._get_section(node)
        section[name] = str(value)



"""OS Abstraction Layer for all file based functions"""

import os
import platform
import shutil
import stat
import subprocess
import tempfile
from pathlib import Path
from shutil import rmtree
from typing import List

from jinja2 import Template

from conan_explorer import INVALID_PATH, PKG_NAME, asset_path
from conan_explorer.app.logger import Logger

WIN_EXE_FILE_TYPES = [".cmd", ".com", ".bat", ".ps1", ".exe"]


def str2bool(value: str) -> bool:
    """Own impl. isntead of distutils.util.strtobool
    because distutils will be deprecated"""
    value = value.lower()
    if value in {"yes", "true", "y", "1"}:
        return True
    if value in {"no", "false", "n", "0"}:
        return False
    return False


def is_windows_11():
    """Main version number is still 10 - thanks MS!"""
    from packaging import version

    if platform.system() == "Windows" and version.parse(platform.version()) >= version.parse(
        "10.0.22000"
    ):
        return True
    return False


def run_file(file_path: Path, is_console_app: bool, args: str):
    """Decide, if a file should be opened or executed and call the appropriate method"""
    if not file_path.is_file():
        return
    try:
        if is_file_executable(file_path):
            execute_app(file_path, is_console_app, args)
        else:
            open_file(file_path)
    except Exception:
        Logger().error(f"Error while executing {str(file_path)} with args: {args}.")


def open_in_file_manager(file_path: Path):
    """Show file in file manager."""
    try:
        if platform.system() == "Linux":
            # no standardized select functionailty.
            # xdg-open on a dir will open the folder in the default file explorer.
            dir_to_view = file_path.parent if file_path.is_file() else file_path
            return subprocess.Popen(("xdg-open", str(dir_to_view)))
        elif platform.system() == "Windows":
            # select switch for highlighting
            creationflags = subprocess.CREATE_NO_WINDOW  # available since 3.7
            return subprocess.Popen(
                "explorer /select," + str(file_path), creationflags=creationflags
            )
    except Exception as e:
        Logger().error(f"Can't show path in file-manager: {str(e)}")


def open_cmd_in_path(directory_path: Path) -> int:
    """Open a terminal in the selected folder."""
    try:
        if not directory_path.is_dir():
            Logger().error(f"Invalid directory: {str(directory_path)}")
            return -1
        if platform.system() == "Linux":
            return execute_cmd(
                [
                    "x-terminal-emulator",
                    "-e",
                    '"',
                    "cd",
                    f"{str(directory_path)}",
                    "&&",
                    "bash",
                    '"',
                ],
                is_console_app=False,
                shell_linux=True,
            )
        elif platform.system() == "Windows":
            cmd_path = shutil.which("cmd")
            if cmd_path:
                return execute_app(Path(cmd_path), True, f"/k cd {str(directory_path)}")
        return -2
    except Exception as e:
        Logger().error(f"Can't open cmd in path: {str(e)}")
        return -3


def is_file_executable(file_path: Path) -> bool:
    """Checking execution mode is ok on linux, but not enough on windows,
    since every file with an associated program has this flag.
    Use custom pathext like list to determine executable file extensions."""
    is_executable = False
    if platform.system() == "Linux":
        if os.access(str(file_path), os.X_OK):
            is_executable = True
    elif platform.system() == "Windows":
        # don't use PATHEXT - some programs write other filetypes like .py in it...
        path_exts = WIN_EXE_FILE_TYPES
        if file_path.suffix in path_exts:
            is_executable = True
    return is_executable


def execute_app(executable: Path, is_console_app: bool, args: str) -> int:
    """
    Executes an application with args and optionally spawns a new shell.
    Returns the pid of the new process.
    """
    if executable.absolute().is_file():
        cmd = [str(executable)]
        if args:
            cmd += args.strip().split(" ")
        return execute_cmd(cmd, is_console_app)
    Logger().warning(f"No executable {str(executable)} to start.")
    return 0


def execute_cmd(cmd: List[str], is_console_app: bool, shell_linux=False) -> int:  # pid
    """Generic process execute method. Returns pid."""
    command_path = Path(cmd[0]).parent
    try:
        # Linux call errors on creationflags argument, so the calls must be separated
        if platform.system() == "Windows":
            creationflags = 0
            if is_console_app:
                creationflags = subprocess.CREATE_NEW_CONSOLE
                cmd = [generate_launch_script(cmd)]
            # don't use 'executable' arg, because shell scripts won't work correctly
            proc = subprocess.Popen(cmd, creationflags=creationflags, cwd=str(command_path))
            return proc.pid
        elif platform.system() == "Linux":
            if is_console_app:
                # Sadly, there is no default way to do this, because of the miriad terminal
                # emulators available. Use the default distro emulator through
                # x-terminal-emulator.
                cmd = ["x-terminal-emulator", "-e"] + [generate_launch_script(cmd)]
            proc = subprocess.Popen(cmd, cwd=str(command_path), shell=shell_linux)
            return proc.pid
        return 0
    except Exception as e:
        Logger().error(f"Can't execute cmd: {str(e)}")
        return 0


def generate_launch_script(cmd: List[str]) -> str:
    """
    Generate a launch script for shell cmd, which will halt on error and not close
    the terminal.
    """
    launch_templ_file = ""

    if platform.system() == "Windows":
        launch_templ_file = "launch.bat.in"
        temp_fd, temp_path_str = tempfile.mkstemp(".bat", prefix=PKG_NAME, text=True)
    elif platform.system() == "Linux":
        launch_templ_file = "launch.sh.in"
        temp_fd, temp_path_str = tempfile.mkstemp(".sh", prefix=PKG_NAME, text=True)
    else:
        Logger().warning("Not supported OS.")
        return ""
    launch_templ_path = asset_path / launch_templ_file
    with open(launch_templ_path, "r") as fd:
        launch_template = Template(fd.read())
    launch_content = launch_template.render(COMMAND=" ".join(cmd))
    with os.fdopen(temp_fd, "w") as f:
        f.write(launch_content)
    if platform.system() == "Linux":
        os.system(f"chmod +x {temp_path_str}")
    return temp_path_str


def open_file(file: Path):
    """Open files with their associated programs"""
    try:
        if file.absolute().is_file():
            if platform.system() == "Windows":
                os.startfile(str(file))
            elif platform.system() == "Linux":
                subprocess.Popen(("xdg-open", str(file)))
    except Exception as e:
        Logger().error(f"Can't open file: {str(e)}")
        return 0


def delete_path(dst: Path):
    """
    Delete file or (non-empty) folder recursively.
    Exceptions will be caught and message logged to stdout.
    """

    try:
        if dst.is_file():
            os.remove(dst)
        elif dst.is_dir():

            def rm_dir_readonly(func, path, _):
                "Clear the readonly bit and reattempt the removal"
                os.chmod(path, stat.S_IWRITE)
                func(path)

            rmtree(str(dst), onerror=rm_dir_readonly)
    except Exception as e:
        Logger().warning(f"Can't delete {str(dst)}: {str(e)}")


def get_folder_size_mb(folder_path: Path):
    if not folder_path.exists():
        return 0
    # exists check is needed for soft links under windows
    return sum(file.stat().st_size for file in folder_path.rglob("*") if file.exists()) / pow(
        1024, 2
    )


def copy_path_with_overwrite(src: Path, dst: Path):
    """
    Copy files/directories while overwriting possible files and adding missing ones.
    Directories will be copied from under source, so you may need to add the original
    folder name, if you want that!
    Exceptions will be caught and message logged to stdout.
    """
    from shutil import copy2, copytree

    try:
        dst.parent.mkdir(parents=True, exist_ok=True)
        if src.is_file():
            copy2(str(src), str(dst))
        else:
            copytree(str(src), str(dst), dirs_exist_ok=True)
    except Exception as e:
        Logger().warning(f"Can't copy {str(src)} to {str(dst)}: {str(e)}")


def calc_paste_same_dir_name(dst: Path, index=1):
    """
    Create a name for a file like /file.txt -> /file (2).txt.
    It will find the next empty number.
    If a file with a higher number exists it will ignore it.
    """
    if dst.exists():
        new_path = dst.with_name(f"{dst.stem} ({str(index+1)}){dst.suffix}")
        possible_path = calc_paste_same_dir_name(new_path, index + 1)
        if possible_path == Path(INVALID_PATH):
            return new_path
        else:
            return dst
    else:
        if index == 1:  # if file does not exist
            return dst
        return Path(INVALID_PATH)


def get_default_file_editor():
    if platform.system() == "Windows":
        editor_executable = find_program_in_windows(
            "Notepad++", partial_match=True, key_to_find="DisplayIcon"
        )
        if Path(editor_executable).exists():
            return editor_executable
        return "notepad.exe"
    else:
        return "gedit"  # distro dependent, but make something


def find_program_in_windows(
    app_name: str, partial_match=False, key_to_find="InstallLocation"
) -> str:
    if platform.system() != "Windows":
        return ""

    import winreg

    arch_keys = {winreg.KEY_WOW64_32KEY, winreg.KEY_WOW64_64KEY}
    for arch_key in arch_keys:
        key = winreg.OpenKey(
            winreg.HKEY_LOCAL_MACHINE,
            r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall",
            0,
            winreg.KEY_READ | arch_key,
        )
        for i in range(0, winreg.QueryInfoKey(key)[0]):
            sub_key_name = winreg.EnumKey(key, i)
            sub_key = winreg.OpenKey(key, sub_key_name)
            try:
                current_app_name = winreg.QueryValueEx(sub_key, "DisplayName")[0]
                if partial_match and app_name in current_app_name:
                    location = winreg.QueryValueEx(sub_key, key_to_find)[0]
                    return location
                if app_name == current_app_name:
                    location = winreg.QueryValueEx(sub_key, key_to_find)[0]
                    return location
            except OSError:
                pass
            finally:
                sub_key.Close()
    return ""


def check_for_wayland() -> bool:
    if platform.system() != "Linux":
        return False
    if os.getenv("XDG_SESSION_TYPE", "").lower() == "wayland" or os.getenv("WAYLAND_DISPLAY"):
        Logger().debug("Found XDG_SESSION_TYPE==wayland or WAYLAND_DISPLAY")
        return True
    return False



from typing import Any, List, Optional, Union

from PySide6.QtCore import QModelIndex, QPersistentModelIndex, Qt
from typing_extensions import override

import conan_explorer.app as app  # using global module pattern
from conan_explorer.conan_wrapper.types import EditablePkg
from conan_explorer.ui.common import TreeModel, TreeModelItem


class EditableModelItemRoot(TreeModelItem):
    def __init__(self, data: List[str], parent=None, lazy_loading=False):
        self.child_items: List[EditableModelItem] = []  # type: ignore
        super().__init__(data, parent, lazy_loading)


class EditableModelItem(TreeModelItem):
    def __init__(self, name: str, path: str, output: str, parent=None, lazy_loading=False):
        super().__init__([name, path, output], parent, lazy_loading=lazy_loading)

    @property
    def name(self) -> str:
        return self.item_data[0]

    @name.setter
    def name(self, value: str):
        self.item_data[0] = value

    @property
    def path(self) -> str:
        return self.item_data[1]

    @path.setter
    def path(self, value: str):
        self.item_data[1] = value

    @property
    def output(self) -> str:
        return self.item_data[2]

    @output.setter
    def output(self, value: str):
        self.item_data[2] = value


class EditableModel(TreeModel):
    def __init__(self):
        super(EditableModel, self).__init__(checkable=True)
        self.root_item = EditableModelItemRoot(["Name", "Path", "Output folder"])  # "Layout",
        self.setup_model_data()

    def setup_model_data(self):
        editables = app.conan_api.get_editable_references()
        self.clear_items()
        self.beginResetModel()
        for editable_ref in editables:
            path = app.conan_api.get_editables_package_path(editable_ref)
            output = app.conan_api.get_editables_output_folder(editable_ref)
            output_str = str(output) if output else ""
            remote_item = EditableModelItem(
                str(editable_ref), str(path), output_str, self.root_item
            )
            self.root_item.append_child(remote_item)
        self.endResetModel()

    def add(self, editable: EditablePkg):
        args = (editable.conan_ref, editable.path, str(editable.output_folder))
        if app.conan_api.add_editable(*args):
            super().add_item(EditableModelItem(*args))
            return True
        return False

    def remove(self, editable: EditablePkg):
        if app.conan_api.remove_editable(editable.conan_ref):
            index = self.get_index_from_ref(editable.conan_ref)
            item: EditableModelItem = index.internalPointer()  # type: ignore
            super().remove_item(item)
            return True
        return False

    @override
    def data(self, index: Union[QModelIndex, QPersistentModelIndex], role: int) -> Any:
        if not index.isValid():
            return None
        item: EditableModelItem = index.internalPointer()  # type: ignore
        if role == Qt.ItemDataRole.DisplayRole:
            try:
                return item.data(index.column())
            except Exception:
                return ""
        return None

    @override
    def rowCount(self, parent=None):
        return self.root_item.child_count()

    def get_index_from_ref(self, conan_ref: str) -> Optional[QModelIndex]:
        for ref in self.root_item.child_items:  # type: ignore
            ref: EditableModelItem
            if ref.name == conan_ref:
                return self.get_index_from_item(ref)
        return None



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'diff.ui'
##
## Created by: Qt User Interface Compiler version 6.7.2
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QAbstractButton, QAbstractItemView, QApplication, QDialog,
    QDialogButtonBox, QLabel, QListView, QListWidget,
    QListWidgetItem, QSizePolicy, QSplitter, QTextBrowser,
    QVBoxLayout, QWidget)

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        if not Dialog.objectName():
            Dialog.setObjectName(u"Dialog")
        Dialog.resize(1000, 500)
        Dialog.setSizeGripEnabled(True)
        self.verticalLayout_4 = QVBoxLayout(Dialog)
        self.verticalLayout_4.setObjectName(u"verticalLayout_4")
        self.splitter_2 = QSplitter(Dialog)
        self.splitter_2.setObjectName(u"splitter_2")
        self.splitter_2.setOrientation(Qt.Horizontal)
        self.pkgs_list_widget = QListWidget(self.splitter_2)
        self.pkgs_list_widget.setObjectName(u"pkgs_list_widget")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pkgs_list_widget.sizePolicy().hasHeightForWidth())
        self.pkgs_list_widget.setSizePolicy(sizePolicy)
        self.pkgs_list_widget.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.pkgs_list_widget.setTabKeyNavigation(True)
        self.pkgs_list_widget.setSelectionBehavior(QAbstractItemView.SelectItems)
        self.pkgs_list_widget.setProperty("isWrapping", False)
        self.pkgs_list_widget.setViewMode(QListView.ListMode)
        self.pkgs_list_widget.setWordWrap(False)
        self.pkgs_list_widget.setSelectionRectVisible(True)
        self.splitter_2.addWidget(self.pkgs_list_widget)
        self.widget = QWidget(self.splitter_2)
        self.widget.setObjectName(u"widget")
        self.verticalLayout_3 = QVBoxLayout(self.widget)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.splitter = QSplitter(self.widget)
        self.splitter.setObjectName(u"splitter")
        self.splitter.setOrientation(Qt.Horizontal)
        self.widget1 = QWidget(self.splitter)
        self.widget1.setObjectName(u"widget1")
        self.verticalLayout = QVBoxLayout(self.widget1)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.label = QLabel(self.widget1)
        self.label.setObjectName(u"label")

        self.verticalLayout.addWidget(self.label)

        self.left_text_browser = QTextBrowser(self.widget1)
        self.left_text_browser.setObjectName(u"left_text_browser")

        self.verticalLayout.addWidget(self.left_text_browser)

        self.splitter.addWidget(self.widget1)
        self.widget2 = QWidget(self.splitter)
        self.widget2.setObjectName(u"widget2")
        self.verticalLayout_2 = QVBoxLayout(self.widget2)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.label_2 = QLabel(self.widget2)
        self.label_2.setObjectName(u"label_2")

        self.verticalLayout_2.addWidget(self.label_2)

        self.right_text_browser = QTextBrowser(self.widget2)
        self.right_text_browser.setObjectName(u"right_text_browser")

        self.verticalLayout_2.addWidget(self.right_text_browser)

        self.splitter.addWidget(self.widget2)

        self.verticalLayout_3.addWidget(self.splitter)

        self.button_box = QDialogButtonBox(self.widget)
        self.button_box.setObjectName(u"button_box")
        self.button_box.setOrientation(Qt.Horizontal)
        self.button_box.setStandardButtons(QDialogButtonBox.Ok)

        self.verticalLayout_3.addWidget(self.button_box)

        self.splitter_2.addWidget(self.widget)

        self.verticalLayout_4.addWidget(self.splitter_2)


        self.retranslateUi(Dialog)

        QMetaObject.connectSlotsByName(Dialog)
    # setupUi

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QCoreApplication.translate("Dialog", u"Dialog", None))
        self.label.setText(QCoreApplication.translate("Dialog", u"Reference (*):", None))
        self.label_2.setText(QCoreApplication.translate("Dialog", u"Compare to:", None))
    # retranslateUi




from dataclasses import dataclass
from typing import TYPE_CHECKING, Optional

from PySide6.QtCore import Qt, Signal, SignalInstance
from PySide6.QtWidgets import QSizePolicy, QWidget
from typing_extensions import override

from ..fluent_window.fluent_window import ThemedWidget

if TYPE_CHECKING:
    from conan_explorer.ui.fluent_window import FluentWindow
    from conan_explorer.ui.main_window import BaseSignals


@dataclass
class PluginDescription:
    """Plugin meta information for runtime"""

    name: str  # Display name, also used as page title
    version: str
    author: str
    icon: str  # left menu icon
    import_path: str  # this path will be placed on python path or a file directly
    plugin_class: str  # class to be loaded from module
    description: str
    side_menu: bool  # will create a side menu, which can be accessed by page_widgets
    conan_versions: str  # spec to restrict the plugin to a conan version (will be greyed out)


class PluginInterfaceV1(ThemedWidget):
    """
    Class to extend the application with custom views.
    """

    # Return a signal, which will be called, when the Ui should load.
    # Connect this to your actual load method.
    # This is used for asynchronous loading.
    load_signal: SignalInstance = Signal()  # type: ignore

    def __init__(
        self,
        parent: QWidget,
        plugin_description: PluginDescription,
        base_signals: Optional["BaseSignals"] = None,
        page_widgets: Optional["FluentWindow.PageStore"] = None,
    ) -> None:
        ThemedWidget.__init__(self, parent)
        self._base_signals = base_signals
        self._page_widgets = page_widgets
        # save PluginDescription to query data from outside
        self.plugin_description = plugin_description
        size_policy = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Preferred)
        size_policy.setVerticalStretch(0)
        self.setSizePolicy(size_policy)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
